@@ -3,7 +3,7 @@ from typing import Optional
 
 import click
 
-from chinilla.util.config import lock_and_load_config, save_config, str2bool
+from chinilla.util.config import load_defaults_for_missing_services, lock_and_load_config, save_config, str2bool
 
 
 def configure(
@@ -22,8 +22,12 @@ def configure(
     crawler_minimum_version_count: Optional[int],
     seeder_domain_name: str,
     seeder_nameserver: str,
+    enable_data_server: str = "",
 ):
-    with lock_and_load_config(root_path, "config.yaml") as config:
+    config_yaml = "config.yaml"
+    with lock_and_load_config(root_path, config_yaml, fill_missing_services=True) as config:
+        config.update(load_defaults_for_missing_services(config=config, config_name=config_yaml))
+
         change_made = False
         if set_node_introducer:
             try:
@@ -92,6 +96,13 @@ def configure(
             config["full_node"]["target_peer_count"] = int(set_peer_count)
             print("Target peer count updated")
             change_made = True
+        if enable_data_server:
+            config["data_layer"]["run_server"] = str2bool(enable_data_server)
+            if str2bool(enable_data_server):
+                print("Data Server enabled.")
+            else:
+                print("Data Server disabled.")
+            change_made = True
         if testnet:
             if testnet == "true" or testnet == "t":
                 print("Setting Testnet")
@@ -101,6 +112,12 @@ def configure(
                 bootstrap_peers = ["testnet10-node.chinilla.com"]
                 testnet = "testnet10"
                 config["full_node"]["port"] = int(testnet_port)
+                if config["full_node"]["introducer_peer"] is None:
+                    config["full_node"]["introducer_peer"] = {}
+                assert config["full_node"]["introducer_peer"] is not None  # mypy
+                if config["wallet"]["introducer_peer"] is None:
+                    config["wallet"]["introducer_peer"] = {}
+                assert config["wallet"]["introducer_peer"] is not None  # mypy
                 config["full_node"]["introducer_peer"]["port"] = int(testnet_port)
                 config["farmer"]["full_node_peer"]["port"] = int(testnet_port)
                 config["timelord"]["full_node_peer"]["port"] = int(testnet_port)
@@ -120,6 +137,7 @@ def configure(
                 config["ui"]["selected_network"] = testnet
                 config["introducer"]["selected_network"] = testnet
                 config["wallet"]["selected_network"] = testnet
+                config["data_layer"]["selected_network"] = testnet
 
                 if "seeder" in config:
                     config["seeder"]["port"] = int(testnet_port)
@@ -157,6 +175,7 @@ def configure(
                 config["ui"]["selected_network"] = net
                 config["introducer"]["selected_network"] = net
                 config["wallet"]["selected_network"] = net
+                config["data_layer"]["selected_network"] = net
 
                 if "seeder" in config:
                     config["seeder"]["port"] = int(vanillanet_port)
@@ -194,7 +213,7 @@ def configure(
             save_config(root_path, "config.yaml", config)
 
 
-@click.command("configure", short_help="Modify configuration")
+@click.command("configure", short_help="Modify configuration", no_args_is_help=True)
 @click.option(
     "--testnet",
     "-t",
@@ -254,6 +273,11 @@ def configure(
     help="configures the seeder nameserver setting. Ex: `example.com.`",
     type=str,
 )
+@click.option(
+    "--enable-data-server",
+    help="Enable or disable data propagation server for your data layer",
+    type=click.Choice(["true", "t", "false", "f"]),
+)
 @click.pass_context
 def configure_cmd(
     ctx,
@@ -271,6 +295,7 @@ def configure_cmd(
     crawler_minimum_version_count,
     seeder_domain_name,
     seeder_nameserver,
+    enable_data_server,
 ):
     configure(
         ctx.obj["root_path"],
@@ -288,4 +313,5 @@ def configure_cmd(
         crawler_minimum_version_count,
         seeder_domain_name,
         seeder_nameserver,
+        enable_data_server,
     )
