@@ -26,7 +26,7 @@ from chinilla.util.config import (
 )
 from chinilla.util.db_version import set_db_version
 from chinilla.util.keychain import Keychain
-from chinilla.util.path import mkdir, path_from_root
+from chinilla.util.path import path_from_root
 from chinilla.util.ssl_check import (
     DEFAULT_PERMISSIONS_CERT_FILE,
     DEFAULT_PERMISSIONS_KEY_FILE,
@@ -44,8 +44,17 @@ from chinilla.wallet.derive_keys import (
 )
 from chinilla.cmds.configure import configure
 
-private_node_names: List[str] = ["full_node", "wallet", "farmer", "harvester", "timelord", "crawler", "daemon"]
-public_node_names: List[str] = ["full_node", "wallet", "farmer", "introducer", "timelord"]
+_all_private_node_names: List[str] = [
+    "full_node",
+    "wallet",
+    "farmer",
+    "harvester",
+    "timelord",
+    "crawler",
+    "data_layer",
+    "daemon",
+]
+_all_public_node_names: List[str] = ["full_node", "wallet", "farmer", "introducer", "timelord", "data_layer"]
 
 
 def dict_add_new_default(updated: Dict, default: Dict, do_not_migrate_keys: Dict[str, Any]):
@@ -165,7 +174,7 @@ def check_keys(new_root: Path, keychain: Optional[Keychain] = None) -> None:
 def copy_files_rec(old_path: Path, new_path: Path):
     if old_path.is_file():
         print(f"{new_path}")
-        mkdir(new_path.parent)
+        new_path.parent.mkdir(parents=True, exist_ok=True)
         shutil.copy(old_path, new_path)
     elif old_path.is_dir():
         for old_path_child in old_path.iterdir():
@@ -216,6 +225,9 @@ def create_all_ssl(
     *,
     private_ca_crt_and_key: Optional[Tuple[bytes, bytes]] = None,
     node_certs_and_keys: Optional[Dict[str, Dict]] = None,
+    private_node_names: List[str] = _all_private_node_names,
+    public_node_names: List[str] = _all_public_node_names,
+    overwrite: bool = True,
 ):
     # remove old key and crt
     config_dir = root_path / "config"
@@ -237,7 +249,13 @@ def create_all_ssl(
     chinilla_ca_crt, chinilla_ca_key = get_chinilla_ca_crt_key()
     chinilla_ca_crt_path = ca_dir / "chinilla_ca.crt"
     chinilla_ca_key_path = ca_dir / "chinilla_ca.key"
-    write_ssl_cert_and_key(chinilla_ca_crt_path, chinilla_ca_crt, chinilla_ca_key_path, chinilla_ca_key)
+    write_ssl_cert_and_key(
+        chinilla_ca_crt_path,
+        chinilla_ca_crt,
+        chinilla_ca_key_path,
+        chinilla_ca_key,
+        overwrite=overwrite,
+    )
 
     # If Private CA crt/key are passed-in, write them out
     if private_ca_crt_and_key is not None:
@@ -252,7 +270,13 @@ def create_all_ssl(
         ca_key = private_ca_key_path.read_bytes()
         ca_crt = private_ca_crt_path.read_bytes()
         generate_ssl_for_nodes(
-            ssl_dir, ca_crt, ca_key, prefix="private", nodes=private_node_names, node_certs_and_keys=node_certs_and_keys
+            ssl_dir,
+            ca_crt,
+            ca_key,
+            prefix="private",
+            nodes=private_node_names,
+            node_certs_and_keys=node_certs_and_keys,
+            overwrite=overwrite,
         )
     else:
         # This is entered when user copied over private CA
@@ -260,7 +284,13 @@ def create_all_ssl(
         ca_key = private_ca_key_path.read_bytes()
         ca_crt = private_ca_crt_path.read_bytes()
         generate_ssl_for_nodes(
-            ssl_dir, ca_crt, ca_key, prefix="private", nodes=private_node_names, node_certs_and_keys=node_certs_and_keys
+            ssl_dir,
+            ca_crt,
+            ca_key,
+            prefix="private",
+            nodes=private_node_names,
+            node_certs_and_keys=node_certs_and_keys,
+            overwrite=overwrite,
         )
 
     chinilla_ca_crt, chinilla_ca_key = get_chinilla_ca_crt_key()
@@ -499,7 +529,7 @@ def chinilla_init(
             db_path_replaced = new_db_path.replace("CHALLENGE", config["selected_network"])
             db_path = path_from_root(root_path, db_path_replaced)
 
-            mkdir(db_path.parent)
+            db_path.parent.mkdir(parents=True, exist_ok=True)
             with sqlite3.connect(db_path) as connection:
                 set_db_version(connection, 1)
 
@@ -509,10 +539,14 @@ def chinilla_init(
         config = load_config(root_path, "config.yaml")["full_node"]
         db_path_replaced = config["database_path"].replace("CHALLENGE", config["selected_network"])
         db_path = path_from_root(root_path, db_path_replaced)
-        mkdir(db_path.parent)
-
-        with sqlite3.connect(db_path) as connection:
-            set_db_version(connection, 2)
+        db_path.parent.mkdir(parents=True, exist_ok=True)
+        try:
+            # create new v2 db file
+            with sqlite3.connect(db_path) as connection:
+                set_db_version(connection, 2)
+        except sqlite3.OperationalError:
+            # db already exists, so we're good
+            pass
 
     print("")
     print("To see your keys, run 'chinilla keys show --show-mnemonic-seed'")
