@@ -1,3 +1,4 @@
+import logging
 import os
 import sys
 
@@ -7,7 +8,10 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from chinilla.consensus.coinbase import create_puzzlehash_for_pk
+from chinilla.daemon.client import connect_to_daemon_and_validate
+from chinilla.daemon.keychain_proxy import KeychainProxy, connect_to_keychain_and_validate, wrap_local_keychain
 from chinilla.util.bech32m import encode_puzzle_hash
+from chinilla.util.errors import KeychainNotSet
 from chinilla.util.config import load_config
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
 from chinilla.util.ints import uint32
@@ -183,48 +187,90 @@ def verify(message: str, public_key: str, signature: str):
     print(AugSchemeMPL.verify(public_key, messageBytes, signature))
 
 
-def migrate_keys():
+async def migrate_keys(root_path: Path, forced: bool = False) -> bool:
     from chinilla.util.keyring_wrapper import KeyringWrapper
     from chinilla.util.misc import prompt_yes_no
 
+    deprecation_message = (
+        "\nLegacy keyring support is deprecated and will be removed in an upcoming version. "
+        "You need to migrate your keyring to continue using Chinilla.\n"
+    )
+
     # Check if the keyring needs a full migration (i.e. if it's using the old keyring)
     if Keychain.needs_migration():
-        KeyringWrapper.get_shared_instance().migrate_legacy_keyring_interactive()
+        print(deprecation_message)
+        return await KeyringWrapper.get_shared_instance().migrate_legacy_keyring_interactive()
     else:
-        keys_to_migrate, legacy_keyring = Keychain.get_keys_needing_migration()
-        if len(keys_to_migrate) > 0 and legacy_keyring is not None:
+        already_checked_marker = KeyringWrapper.get_shared_instance().keys_root_path / ".checked_legacy_migration"
+        if forced and already_checked_marker.exists():
+            return True
+
+        log = logging.getLogger("migrate_keys")
+        config = load_config(root_path, "config.yaml")
+        # Connect to the daemon here first to see if ts running since `connect_to_keychain_and_validate` just tries to
+        # connect forever if it's not up.
+        keychain_proxy: Optional[KeychainProxy] = None
+        daemon = await connect_to_daemon_and_validate(root_path, config, quiet=True)
+        if daemon is not None:
+            await daemon.close()
+            keychain_proxy = await connect_to_keychain_and_validate(root_path, log)
+        if keychain_proxy is None:
+            keychain_proxy = wrap_local_keychain(Keychain(), log=log)
+
+        try:
+            legacy_keyring = Keychain(force_legacy=True)
+            all_sks = await keychain_proxy.get_all_private_keys()
+            all_legacy_sks = legacy_keyring.get_all_private_keys()
+            set_legacy_sks = {str(x[0]) for x in all_legacy_sks}
+            set_sks = {str(x[0]) for x in all_sks}
+            missing_legacy_keys = set_legacy_sks - set_sks
+            keys_to_migrate = [x for x in all_legacy_sks if str(x[0]) in missing_legacy_keys]
+        except KeychainNotSet:
+            keys_to_migrate = []
+
+        if len(keys_to_migrate) > 0:
+            print(deprecation_message)
             print(f"Found {len(keys_to_migrate)} key(s) that need migration:")
             for key, _ in keys_to_migrate:
                 print(f"Fingerprint: {key.get_g1().get_fingerprint()}")
 
             print()
-            response = prompt_yes_no("Migrate these keys? (y/n) ")
-            if response:
-                keychain = Keychain()
-                for sk, seed_bytes in keys_to_migrate:
-                    mnemonic = bytes_to_mnemonic(seed_bytes)
-                    keychain.add_private_key(mnemonic, "")
-                    fingerprint = sk.get_g1().get_fingerprint()
-                    print(f"Added private key with public key fingerprint {fingerprint}")
+            if not prompt_yes_no("Migrate these keys?"):
+                await keychain_proxy.close()
+                print("Migration aborted, can't run any chinilla commands.")
+                return False
 
-                print(f"Migrated {len(keys_to_migrate)} key(s)")
+            for sk, seed_bytes in keys_to_migrate:
+                mnemonic = bytes_to_mnemonic(seed_bytes)
+                await keychain_proxy.add_private_key(mnemonic, "")
+                fingerprint = sk.get_g1().get_fingerprint()
+                print(f"Added private key with public key fingerprint {fingerprint}")
 
-                print("Verifying migration results...", end="")
-                if Keychain.verify_keys_present(keys_to_migrate):
-                    print(" Verified")
-                    print()
-                    response = prompt_yes_no("Remove key(s) from old keyring? (y/n) ")
-                    if response:
-                        legacy_keyring.delete_keys(keys_to_migrate)
-                        print(f"Removed {len(keys_to_migrate)} key(s) from old keyring")
-                    print("Migration complete")
-                else:
-                    print(" Failed")
-                    sys.exit(1)
-        else:
+            print(f"Migrated {len(keys_to_migrate)} key(s)")
+
+            print("Verifying migration results...", end="")
+            all_sks = await keychain_proxy.get_all_private_keys()
+            await keychain_proxy.close()
+            set_sks = {str(x[0]) for x in all_sks}
+            keys_present = set_sks.issuperset(set(map(lambda x: str(x[0]), keys_to_migrate)))
+            if keys_present:
+                print(" Verified")
+                print()
+                response = prompt_yes_no("Remove key(s) from old keyring (recommended)?")
+                if response:
+                    legacy_keyring.delete_keys(keys_to_migrate)
+                    print(f"Removed {len(keys_to_migrate)} key(s) from old keyring")
+                print("Migration complete")
+            else:
+                print(" Failed")
+                return False
+            return True
+        elif not forced:
             print("No keys need migration")
-
-        Keychain.mark_migration_checked_for_current_version()
+        if already_checked_marker.parent.exists():
+            already_checked_marker.touch()
+        await keychain_proxy.close()
+    return True
 
 
 def _clear_line_part(n: int):
