@@ -9,7 +9,8 @@ from chinilla.wallet.util.wallet_types import WalletType
 
 
 @click.group("wallet", short_help="Manage your wallet")
-def wallet_cmd() -> None:
+@click.pass_context
+def wallet_cmd(ctx: click.Context) -> None:
     pass
 
 
@@ -28,7 +29,8 @@ def wallet_cmd() -> None:
 def get_transaction_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int, tx_id: str, verbose: int) -> None:
     extra_params = {"id": id, "tx_id": tx_id, "verbose": verbose}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_transaction
+    from .wallet_funcs import get_transaction
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_transaction))
 
@@ -108,7 +110,8 @@ def get_transactions_cmd(
     }
 
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_transactions
+    from .wallet_funcs import get_transactions
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_transactions))
 
@@ -175,7 +178,8 @@ def send_cmd(
         "min_coin_amount": min_coin_amount,
     }
     import asyncio
-    from .wallet_funcs import execute_with_wallet, send
+    from .wallet_funcs import send
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, send))
 
@@ -198,7 +202,8 @@ def send_cmd(
 )
 def show_cmd(wallet_rpc_port: Optional[int], fingerprint: int, wallet_type: Optional[str]) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, print_balances
+    from .wallet_funcs import print_balances
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     args: Dict[str, Any] = {}
     if wallet_type is not None:
@@ -229,7 +234,8 @@ def show_cmd(wallet_rpc_port: Optional[int], fingerprint: int, wallet_type: Opti
 def get_address_cmd(wallet_rpc_port: Optional[int], id, fingerprint: int, new_address: bool) -> None:
     extra_params = {"id": id, "new_address": new_address}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_address
+    from .wallet_funcs import get_address
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_address))
 
@@ -249,7 +255,8 @@ def get_address_cmd(wallet_rpc_port: Optional[int], id, fingerprint: int, new_ad
 def delete_unconfirmed_transactions_cmd(wallet_rpc_port: Optional[int], id, fingerprint: int) -> None:
     extra_params = {"id": id}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, delete_unconfirmed_transactions
+    from .wallet_funcs import delete_unconfirmed_transactions
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, delete_unconfirmed_transactions))
 
@@ -266,7 +273,8 @@ def delete_unconfirmed_transactions_cmd(wallet_rpc_port: Optional[int], id, fing
 def get_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int) -> None:
     extra_params: Dict[str, Any] = {}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_derivation_index
+    from .wallet_funcs import get_derivation_index
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_derivation_index))
 
@@ -288,7 +296,8 @@ def get_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int) -
 def update_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int, index: int) -> None:
     extra_params = {"index": index}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, update_derivation_index
+    from .wallet_funcs import update_derivation_index
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, update_derivation_index))
 
@@ -322,7 +331,8 @@ def update_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int
 def add_token_cmd(wallet_rpc_port: Optional[int], asset_id: str, token_name: str, fingerprint: int) -> None:
     extra_params = {"asset_id": asset_id, "token_name": token_name}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, add_token
+    from .wallet_funcs import add_token
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, add_token))
 
@@ -357,7 +367,8 @@ def make_offer_cmd(
 ) -> None:
     extra_params = {"offers": offer, "requests": request, "filepath": filepath, "fee": fee}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, make_offer
+    from .wallet_funcs import make_offer
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, make_offer))
 
@@ -403,7 +414,8 @@ def get_offers_cmd(
         "reverse": reverse,
     }
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_offers
+    from .wallet_funcs import get_offers
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_offers))
 
@@ -425,7 +437,8 @@ def take_offer_cmd(
 ) -> None:
     extra_params = {"file": path_or_hex, "examine_only": examine_only, "fee": fee}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, take_offer
+    from .wallet_funcs import take_offer
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, take_offer))
 
@@ -445,7 +458,8 @@ def take_offer_cmd(
 def cancel_offer_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: str, insecure: bool, fee: str) -> None:
     extra_params = {"id": id, "insecure": insecure, "fee": fee}
     import asyncio
-    from .wallet_funcs import execute_with_wallet, cancel_offer
+    from .wallet_funcs import cancel_offer
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, cancel_offer))
 
@@ -486,7 +500,8 @@ def did_create_wallet_cmd(
     wallet_rpc_port: Optional[int], fingerprint: int, name: Optional[str], amount: Optional[int], fee: Optional[int]
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, create_did_wallet
+    from .wallet_funcs import create_did_wallet
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"amount": amount, "fee": fee, "name": name}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, create_did_wallet))
@@ -505,7 +520,8 @@ def did_create_wallet_cmd(
 @click.option("-n", "--name", help="Set the DID wallet name", type=str, required=True)
 def did_wallet_name_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int, name: str) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, did_set_wallet_name
+    from .wallet_funcs import did_set_wallet_name
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"wallet_id": id, "name": name}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, did_set_wallet_name))
@@ -523,7 +539,8 @@ def did_wallet_name_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: in
 @click.option("-i", "--id", help="Id of the wallet to use", type=int, required=True)
 def did_get_did_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_did
+    from .wallet_funcs import get_did
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"did_wallet_id": id}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_did))
@@ -549,7 +566,8 @@ def nft_wallet_create_cmd(
     wallet_rpc_port: Optional[int], fingerprint: int, did_id: Optional[str], name: Optional[str]
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, create_nft_wallet
+    from .wallet_funcs import create_nft_wallet
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params: Dict[str, Any] = {"did_id": did_id, "name": name}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, create_nft_wallet))
@@ -574,12 +592,8 @@ def nft_wallet_create_cmd(
 @click.option("-mu", "--metadata-uris", help="Comma separated list of metadata URIs", type=str)
 @click.option("-lh", "--license-hash", help="NFT license hash", type=str, default="")
 @click.option("-lu", "--license-uris", help="Comma separated list of license URIs", type=str)
-@click.option(
-    "-st", "--series-total", help="[DEPRECATED] NFT series total number", type=int, default=1, show_default=True
-)
-@click.option("-sn", "--series-number", help="[DEPRECATED] NFT series number", type=int, default=1, show_default=True)
-@click.option("-ec", "--edition-count", help="NFT edition count, defaults to 1", type=int)
-@click.option("-en", "--edition-number", help="NFT edition number, defaults to 1", type=int)
+@click.option("-et", "--edition-total", help="NFT edition total", type=int, show_default=True, default=1)
+@click.option("-en", "--edition-number", help="NFT edition number", show_default=True, default=1, type=int)
 @click.option(
     "-m",
     "--fee",
@@ -610,15 +624,14 @@ def nft_mint_cmd(
     metadata_uris: Optional[str],
     license_hash: Optional[str],
     license_uris: Optional[str],
-    series_total: Optional[int],
-    series_number: Optional[int],
-    edition_count: Optional[int],
+    edition_total: Optional[int],
     edition_number: Optional[int],
     fee: str,
     royalty_percentage_fraction: int,
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, mint_nft
+    from .wallet_funcs import mint_nft
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     if metadata_uris is None:
         metadata_uris_list = []
@@ -630,14 +643,6 @@ def nft_mint_cmd(
     else:
         license_uris_list = [lu.strip() for lu in license_uris.split(",")]
 
-    if not (edition_number and edition_count):
-        if series_number and series_total:
-            print("\nWARNING: Series total(-st) and number(-sn) options are *deprecated*, please use -en and -ec.\n")
-            edition_number = series_number
-            edition_count = series_total
-        else:
-            edition_number = 1
-            edition_count = 1
     extra_params = {
         "wallet_id": id,
         "royalty_address": royalty_address,
@@ -649,7 +654,7 @@ def nft_mint_cmd(
         "metadata_uris": metadata_uris_list,
         "license_hash": license_hash,
         "license_uris": license_uris_list,
-        "edition_count": edition_count,
+        "edition_total": edition_total,
         "edition_number": edition_number,
         "fee": fee,
         "royalty_percentage": royalty_percentage_fraction,
@@ -691,7 +696,8 @@ def nft_add_uri_cmd(
     fee: str,
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, add_uri_to_nft
+    from .wallet_funcs import add_uri_to_nft
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "wallet_id": id,
@@ -734,7 +740,8 @@ def nft_transfer_cmd(
     fee: str,
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, transfer_nft
+    from .wallet_funcs import transfer_nft
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "wallet_id": id,
@@ -757,7 +764,8 @@ def nft_transfer_cmd(
 @click.option("-i", "--id", help="Id of the NFT wallet to use", type=int, required=True)
 def nft_list_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, list_nfts
+    from .wallet_funcs import list_nfts
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"wallet_id": id}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, list_nfts))
@@ -793,7 +801,8 @@ def nft_set_did_cmd(
     fee: str,
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, set_nft_did
+    from .wallet_funcs import set_nft_did
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "wallet_id": id,
@@ -820,7 +829,8 @@ def nft_get_info_cmd(
     nft_coin_id: str,
 ) -> None:
     import asyncio
-    from .wallet_funcs import execute_with_wallet, get_nft_info
+    from .wallet_funcs import get_nft_info
+    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "nft_coin_id": nft_coin_id,
