@@ -18,11 +18,11 @@ def farmer_parent_id(block_height: uint32, genesis_challenge: bytes32) -> bytes3
     return bytes32(genesis_challenge[16:] + block_height.to_bytes(16, "big"))
 
 
-def create_pool_coin(block_height: uint32, puzzle_hash: bytes32, reward: uint64, genesis_challenge: bytes32):
+def create_pool_coin(block_height: uint32, puzzle_hash: bytes32, reward: uint64, genesis_challenge: bytes32) -> Coin:
     parent_id = pool_parent_id(block_height, genesis_challenge)
     return Coin(parent_id, puzzle_hash, reward)
 
 
-def create_farmer_coin(block_height: uint32, puzzle_hash: bytes32, reward: uint64, genesis_challenge: bytes32):
+def create_farmer_coin(block_height: uint32, puzzle_hash: bytes32, reward: uint64, genesis_challenge: bytes32) -> Coin:
     parent_id = farmer_parent_id(block_height, genesis_challenge)
     return Coin(parent_id, puzzle_hash, reward)
