@@ -15,7 +15,7 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, TextIO, Tuple, cast
 
 from chinilla import __version__
-from chinilla.cmds.init_funcs import check_keys, chinilla_init
+from chinilla.cmds.init_funcs import check_keys, chinilla_init, chinilla_full_version_str
 from chinilla.cmds.passphrase_funcs import default_passphrase, using_default_passphrase
 from chinilla.daemon.keychain_server import KeychainServer, keychain_commands
 from chinilla.daemon.windows_signal import kill
@@ -25,16 +25,14 @@ from chinilla.server.server import ssl_context_for_root, ssl_context_for_server
 from chinilla.ssl.create_ssl import get_mozilla_ca_crt
 from chinilla.util.chinilla_logging import initialize_logging
 from chinilla.util.config import load_config
+from chinilla.util.errors import KeychainRequiresMigration, KeychainCurrentPassphraseIsInvalid
 from chinilla.util.json_util import dict_to_json_str
 from chinilla.util.keychain import (
     Keychain,
-    KeyringCurrentPassphraseIsInvalid,
-    KeyringRequiresMigration,
     passphrase_requirements,
-    supports_keyring_passphrase,
     supports_os_passphrase_storage,
 )
-from chinilla.util.path import mkdir
+from chinilla.util.lock import Lockfile, LockfileError
 from chinilla.util.service_groups import validate_service
 from chinilla.util.setproctitle import setproctitle
 from chinilla.util.ws_message import WsRpcMessage, create_payload, format_response
@@ -48,12 +46,6 @@ except ModuleNotFoundError:
     print("Error: Make sure to run . ./activate from the project folder before starting Chinilla.")
     quit()
 
-try:
-    import fcntl
-
-    has_fcntl = True
-except ImportError:
-    has_fcntl = False
 
 log = logging.getLogger(__name__)
 
@@ -91,6 +83,8 @@ class PlotEvent(str, Enum):
 if getattr(sys, "frozen", False):
     name_map = {
         "chinilla": "chinilla",
+        "chinilla_data_layer": "start_data_layer",
+        "chinilla_data_layer_http": "start_data_layer_http",
         "chinilla_wallet": "start_wallet",
         "chinilla_full_node": "start_full_node",
         "chinilla_harvester": "start_harvester",
@@ -207,14 +201,14 @@ class WebSocketServer:
 
     async def stop(self) -> Dict[str, Any]:
         self.cancel_task_safe(self.ping_job)
-        jobs = []
-        for service_name in self.services.keys():
-            jobs.append(kill_service(self.root_path, self.services, service_name))
-        if jobs:
-            await asyncio.wait(jobs)
+        service_names = list(self.services.keys())
+        stop_service_jobs = [kill_service(self.root_path, self.services, s_n) for s_n in service_names]
+        if stop_service_jobs:
+            await asyncio.wait(stop_service_jobs)
         self.services.clear()
         asyncio.create_task(self.exit())
-        return {"success": True}
+        log.info(f"Daemon Server stopping, Services stopped: {service_names}")
+        return {"success": True, "services_stopped": service_names}
 
     async def incoming_connection(self, request):
         ws: WebSocketResponse = web.WebSocketResponse(max_msg_size=self.daemon_max_message_size, heartbeat=30)
@@ -321,7 +315,7 @@ class WebSocketServer:
         if len(data) == 0 and command in commands_with_data:
             response = {"success": False, "error": f'{command} requires "data"'}
         # Keychain commands should be handled by KeychainServer
-        elif command in keychain_commands and supports_keyring_passphrase():
+        elif command in keychain_commands:
             response = await self.keychain_server.handle_command(command, data)
         elif command == "ping":
             response = await ping()
@@ -374,7 +368,6 @@ class WebSocketServer:
         return response
 
     async def keyring_status(self) -> Dict[str, Any]:
-        passphrase_support_enabled: bool = supports_keyring_passphrase()
         can_save_passphrase: bool = supports_os_passphrase_storage()
         user_passphrase_is_set: bool = Keychain.has_master_passphrase() and not using_default_passphrase()
         locked: bool = Keychain.is_keyring_locked()
@@ -386,7 +379,6 @@ class WebSocketServer:
         response: Dict[str, Any] = {
             "success": True,
             "is_keyring_locked": locked,
-            "passphrase_support_enabled": passphrase_support_enabled,
             "can_save_passphrase": can_save_passphrase,
             "user_passphrase_is_set": user_passphrase_is_set,
             "needs_migration": needs_migration,
@@ -410,18 +402,6 @@ class WebSocketServer:
             if Keychain.master_passphrase_is_valid(key, force_reload=True):
                 Keychain.set_cached_master_passphrase(key)
                 success = True
-
-                # Attempt to silently migrate legacy keys if necessary. Non-fatal if this fails.
-                try:
-                    if not Keychain.migration_checked_for_current_version():
-                        self.log.info("Will attempt to migrate legacy keys...")
-                        Keychain.migrate_legacy_keys_silently()
-                        self.log.info("Migration of legacy keys complete.")
-                    else:
-                        self.log.debug("Skipping legacy key migration (previously attempted).")
-                except Exception:
-                    self.log.exception("Failed to migrate keys silently. Run `chinilla keys migrate` manually.")
-
                 # Inform the GUI of keyring status changes
                 self.keyring_status_changed(await self.keyring_status(), "wallet_ui")
             else:
@@ -536,13 +516,12 @@ class WebSocketServer:
             Keychain.set_master_passphrase(
                 current_passphrase,
                 new_passphrase,
-                allow_migration=False,
                 passphrase_hint=passphrase_hint,
                 save_passphrase=save_passphrase,
             )
-        except KeyringRequiresMigration:
+        except KeychainRequiresMigration:
             error = "keyring requires migration"
-        except KeyringCurrentPassphraseIsInvalid:
+        except KeychainCurrentPassphraseIsInvalid:
             error = "current passphrase is invalid"
         except Exception as e:
             tb = traceback.format_exc()
@@ -569,7 +548,7 @@ class WebSocketServer:
 
         try:
             Keychain.remove_master_passphrase(current_passphrase)
-        except KeyringCurrentPassphraseIsInvalid:
+        except KeychainCurrentPassphraseIsInvalid:
             error = "current passphrase is invalid"
         except Exception as e:
             tb = traceback.format_exc()
@@ -891,7 +870,10 @@ class WebSocketServer:
         exclude_final_dir: bool = job["exclude_final_dir"]
         log.info(f"Post-processing plotter job with ID {id}")  # lgtm [py/clear-text-logging-sensitive-data]
         if not exclude_final_dir:
-            add_plot_directory(self.root_path, final_dir)
+            try:
+                add_plot_directory(self.root_path, final_dir)
+            except ValueError as e:
+                log.warning(f"_post_process_plotting_job: {e}")
 
     async def _start_plotting(self, id: str, loop: asyncio.AbstractEventLoop, queue: str = "default"):
         current_process = None
@@ -1182,15 +1164,15 @@ def daemon_launch_lock_path(root_path: Path) -> Path:
     A path to a file that is lock when a daemon is launching but not yet started.
     This prevents multiple instances from launching.
     """
-    return root_path / "run" / "start-daemon.launching"
+    return service_launch_lock_path(root_path, "daemon")
 
 
 def service_launch_lock_path(root_path: Path, service: str) -> Path:
     """
-    A path to a file that is lock when a service is running.
+    A path that is locked when a service is running.
     """
     service_name = service.replace(" ", "-").replace("/", "-")
-    return root_path / "run" / f"{service_name}.lock"
+    return root_path / "run" / service_name
 
 
 def pid_path_for_service(root_path: Path, service: str, id: str = "") -> Path:
@@ -1213,16 +1195,12 @@ def launch_plotter(root_path: Path, service_name: str, service_array: List[str],
     # Swap service name with name of executable
     service_array[0] = service_executable
     startupinfo = None
-    if os.name == "nt":
-        startupinfo = subprocess.STARTUPINFO()  # type: ignore
-        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW  # type: ignore
-
-    # Windows-specific.
-    # If the current process group is used, CTRL_C_EVENT will kill the parent and everyone in the group!
-    try:
-        creationflags: int = subprocess.CREATE_NEW_PROCESS_GROUP  # type: ignore
-    except AttributeError:  # Not on Windows.
-        creationflags = 0
+    creationflags = 0
+    if sys.platform == "win32" or sys.platform == "cygwin":
+        startupinfo = subprocess.STARTUPINFO()
+        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        # If the current process group is used, CTRL_C_EVENT will kill the parent and everyone in the group!
+        creationflags = subprocess.CREATE_NEW_PROCESS_GROUP
 
     plotter_path = plotter_log_path(root_path, id)
 
@@ -1230,7 +1208,7 @@ def launch_plotter(root_path: Path, service_name: str, service_array: List[str],
         if plotter_path.exists():
             plotter_path.unlink()
     else:
-        mkdir(plotter_path.parent)
+        plotter_path.parent.mkdir(parents=True, exist_ok=True)
     outfile = open(plotter_path.resolve(), "w")
     log.info(f"Service array: {service_array}")  # lgtm [py/clear-text-logging-sensitive-data]
     process = subprocess.Popen(
@@ -1244,7 +1222,7 @@ def launch_plotter(root_path: Path, service_name: str, service_array: List[str],
 
     pid_path = pid_path_for_service(root_path, service_name, id)
     try:
-        mkdir(pid_path.parent)
+        pid_path.parent.mkdir(parents=True, exist_ok=True)
         with open(pid_path, "w") as f:
             f.write(f"{process.pid}\n")
     except Exception:
@@ -1263,22 +1241,17 @@ def launch_service(root_path: Path, service_command) -> Tuple[subprocess.Popen,
     # we need to pass on the possibly altered CHINILLA_ROOT
     os.environ["CHINILLA_ROOT"] = str(root_path)
 
-    log.debug(f"Launching service with CHINILLA_ROOT: {os.environ['CHINILLA_ROOT']}")
-
     # Insert proper e
     service_array = service_command.split()
     service_executable = executable_for_service(service_array[0])
     service_array[0] = service_executable
 
-    if service_command == "chinilla_full_node_simulator":
-        # Set the -D/--connect_to_daemon flag to signify that the child should connect
-        # to the daemon to access the keychain
-        service_array.append("-D")
-
     startupinfo = None
-    if os.name == "nt":
-        startupinfo = subprocess.STARTUPINFO()  # type: ignore
-        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW  # type: ignore
+    if sys.platform == "win32" or sys.platform == "cygwin":
+        startupinfo = subprocess.STARTUPINFO()
+        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+
+    log.debug(f"Launching service {service_array} with CHINILLA_ROOT: {os.environ['CHINILLA_ROOT']}")
 
     # CREATE_NEW_PROCESS_GROUP allows graceful shutdown on windows, by CTRL_BREAK_EVENT signal
     if sys.platform == "win32" or sys.platform == "cygwin":
@@ -1289,9 +1262,10 @@ def launch_service(root_path: Path, service_command) -> Tuple[subprocess.Popen,
     process = subprocess.Popen(
         service_array, shell=False, startupinfo=startupinfo, creationflags=creationflags, env=environ_copy
     )
+
     pid_path = pid_path_for_service(root_path, service_command)
     try:
-        mkdir(pid_path.parent)
+        pid_path.parent.mkdir(parents=True, exist_ok=True)
         with open(pid_path, "w") as f:
             f.write(f"{process.pid}\n")
     except Exception:
@@ -1351,29 +1325,6 @@ def is_running(services: Dict[str, subprocess.Popen], service_name: str) -> bool
     return process is not None and process.poll() is None
 
 
-def singleton(lockfile: Path, text: str = "semaphore") -> Optional[TextIO]:
-    """
-    Open a lockfile exclusively.
-    """
-
-    if not lockfile.parent.exists():
-        mkdir(lockfile.parent)
-
-    try:
-        if has_fcntl:
-            f = open(lockfile, "w")
-            fcntl.lockf(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-        else:
-            if lockfile.exists():
-                lockfile.unlink()
-            fd = os.open(lockfile, os.O_CREAT | os.O_EXCL | os.O_RDWR)
-            f = open(fd, "w")
-        f.write(text)
-    except IOError:
-        return None
-    return f
-
-
 async def async_run_daemon(root_path: Path, wait_for_unlock: bool = False) -> int:
     # When wait_for_unlock is true, we want to skip the check_keys() call in chinilla_init
     # since it might be necessary to wait for the GUI to unlock the keyring first.
@@ -1381,7 +1332,6 @@ async def async_run_daemon(root_path: Path, wait_for_unlock: bool = False) -> in
     config = load_config(root_path, "config.yaml")
     setproctitle("chinilla_daemon")
     initialize_logging("daemon", config["logging"], root_path)
-    lockfile = singleton(daemon_launch_lock_path(root_path))
     crt_path = root_path / config["daemon_ssl"]["private_crt"]
     key_path = root_path / config["daemon_ssl"]["private_key"]
     ca_crt_path = root_path / config["private_ssl_ca"]["crt"]
@@ -1398,28 +1348,30 @@ async def async_run_daemon(root_path: Path, wait_for_unlock: bool = False) -> in
     )
     sys.stdout.write("\n" + json_msg + "\n")
     sys.stdout.flush()
-    if lockfile is None:
+    try:
+        with Lockfile.create(daemon_launch_lock_path(root_path), timeout=1):
+            log.info(f"chinilla-blockchain version: {chinilla_full_version_str()}")
+
+            shutdown_event = asyncio.Event()
+
+            ws_server = WebSocketServer(
+                root_path,
+                ca_crt_path,
+                ca_key_path,
+                crt_path,
+                key_path,
+                shutdown_event,
+                run_check_keys_on_unlock=wait_for_unlock,
+            )
+            await ws_server.start()
+            await shutdown_event.wait()
+            log.info("Daemon WebSocketServer closed")
+            # sys.stdout.close()
+            return 0
+    except LockfileError:
         print("daemon: already launching")
         return 2
 
-    shutdown_event = asyncio.Event()
-
-    # TODO: clean this up, ensuring lockfile isn't removed until the listen port is open
-    ws_server = WebSocketServer(
-        root_path,
-        ca_crt_path,
-        ca_key_path,
-        crt_path,
-        key_path,
-        shutdown_event,
-        run_check_keys_on_unlock=wait_for_unlock,
-    )
-    await ws_server.start()
-    await shutdown_event.wait()
-    log.info("Daemon WebSocketServer closed")
-    # sys.stdout.close()
-    return 0
-
 
 def run_daemon(root_path: Path, wait_for_unlock: bool = False) -> int:
     result = asyncio.run(async_run_daemon(root_path, wait_for_unlock))
