@@ -11,12 +11,7 @@ from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 
 import chinilla.server.ws_connection as ws  # lgtm [py/import-and-import-from]
 from chinilla.consensus.constants import ConsensusConstants
-from chinilla.daemon.keychain_proxy import (
-    KeychainProxy,
-    KeychainProxyConnectionFailure,
-    connect_to_keychain_and_validate,
-    wrap_local_keychain,
-)
+from chinilla.daemon.keychain_proxy import KeychainProxy, connect_to_keychain_and_validate, wrap_local_keychain
 from chinilla.plot_sync.delta import Delta
 from chinilla.plot_sync.receiver import Receiver
 from chinilla.pools.pool_config import PoolWalletConfig, add_auth_key, load_pool_config
@@ -42,6 +37,7 @@ from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.bech32m import decode_puzzle_hash
 from chinilla.util.byte_types import hexstr_to_bytes
 from chinilla.util.config import config_path_for_filename, load_config, lock_and_load_config, save_config
+from chinilla.util.errors import KeychainProxyConnectionFailure
 from chinilla.util.hash import std_hash
 from chinilla.util.ints import uint8, uint16, uint32, uint64
 from chinilla.util.keychain import Keychain
@@ -125,7 +121,7 @@ class Farmer:
             else:
                 self.keychain_proxy = await connect_to_keychain_and_validate(self._root_path, self.log)
                 if not self.keychain_proxy:
-                    raise KeychainProxyConnectionFailure("Failed to connect to keychain service")
+                    raise KeychainProxyConnectionFailure()
         return self.keychain_proxy
 
     async def get_all_private_keys(self):
@@ -134,7 +130,11 @@ class Farmer:
 
     async def setup_keys(self) -> bool:
         no_keys_error_str = "No keys exist. Please run 'chinilla keys generate' or open the UI."
-        self.all_root_sks: List[PrivateKey] = [sk for sk, _ in await self.get_all_private_keys()]
+        try:
+            self.all_root_sks: List[PrivateKey] = [sk for sk, _ in await self.get_all_private_keys()]
+        except KeychainProxyConnectionFailure:
+            return False
+
         self._private_keys = [master_sk_to_farmer_sk(sk) for sk in self.all_root_sks] + [
             master_sk_to_pool_sk(sk) for sk in self.all_root_sks
         ]
