@@ -14,18 +14,20 @@ class HintStore:
         self = cls()
         self.db_wrapper = db_wrapper
 
-        async with self.db_wrapper.write_db() as conn:
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
+            log.info("DB: Creating hint store tables and indexes.")
             if self.db_wrapper.db_version == 2:
                 await conn.execute("CREATE TABLE IF NOT EXISTS hints(coin_id blob, hint blob, UNIQUE (coin_id, hint))")
             else:
                 await conn.execute(
                     "CREATE TABLE IF NOT EXISTS hints(id INTEGER PRIMARY KEY AUTOINCREMENT, coin_id blob, hint blob)"
                 )
+            log.info("DB: Creating index hint_index")
             await conn.execute("CREATE INDEX IF NOT EXISTS hint_index on hints(hint)")
         return self
 
     async def get_coin_ids(self, hint: bytes) -> List[bytes32]:
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             cursor = await conn.execute("SELECT coin_id from hints WHERE hint=?", (hint,))
             rows = await cursor.fetchall()
             await cursor.close()
@@ -38,7 +40,7 @@ class HintStore:
         if len(coin_hint_list) == 0:
             return None
 
-        async with self.db_wrapper.write_db() as conn:
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
             if self.db_wrapper.db_version == 2:
                 cursor = await conn.executemany(
                     "INSERT OR IGNORE INTO hints VALUES(?, ?)",
@@ -52,7 +54,7 @@ class HintStore:
             await cursor.close()
 
     async def count_hints(self) -> int:
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             async with conn.execute("select count(*) from hints") as cursor:
                 row = await cursor.fetchone()
 
