@@ -7,7 +7,7 @@ import pathlib
 import random
 from concurrent.futures.process import ProcessPoolExecutor
 import tempfile
-from typing import Dict, IO, List, Optional, Tuple
+from typing import Dict, IO, List, Optional, Tuple, Awaitable
 
 from chinilla.consensus.block_header_validation import validate_finished_header_block
 from chinilla.consensus.block_record import BlockRecord
@@ -42,7 +42,6 @@ from chinilla.util.block_cache import BlockCache
 from chinilla.util.hash import std_hash
 from chinilla.util.ints import uint8, uint32, uint64, uint128
 from chinilla.util.setproctitle import getproctitle, setproctitle
-from chinilla.util.streamable import dataclass_from_dict, recurse_jsonify
 
 log = logging.getLogger(__name__)
 
@@ -566,9 +565,7 @@ class WeightProofHandler:
         if summaries is None:
             log.warning("weight proof failed sub epoch data validation")
             return False, uint32(0)
-        constants, summary_bytes, wp_segment_bytes, wp_recent_chain_bytes = vars_to_bytes(
-            self.constants, summaries, weight_proof
-        )
+        summary_bytes, wp_segment_bytes, wp_recent_chain_bytes = vars_to_bytes(summaries, weight_proof)
         log.info("validate sub epoch challenge segments")
         seed = summaries[-2].get_hash()
         rng = random.Random(seed)
@@ -576,12 +573,14 @@ class WeightProofHandler:
             log.error("failed weight proof sub epoch sample validation")
             return False, uint32(0)
 
-        if not _validate_sub_epoch_segments(constants, rng, wp_segment_bytes, summary_bytes):
+        if not _validate_sub_epoch_segments(self.constants, rng, wp_segment_bytes, summary_bytes):
             return False, uint32(0)
         log.info("validate weight proof recent blocks")
-        if not _validate_recent_blocks(constants, wp_recent_chain_bytes, summary_bytes):
+        success, _ = validate_recent_blocks(self.constants, wp_recent_chain_bytes, summary_bytes)
+        if not success:
             return False, uint32(0)
-        return True, self.get_fork_point(summaries)
+        fork_point, _ = self.get_fork_point(summaries)
+        return True, fork_point
 
     def get_fork_point_no_validations(self, weight_proof: WeightProof) -> Tuple[bool, uint32]:
         log.debug("get fork point skip validations")
@@ -593,36 +592,23 @@ class WeightProofHandler:
         if summaries is None:
             log.warning("weight proof failed to validate sub epoch summaries")
             return False, uint32(0)
-        return True, self.get_fork_point(summaries)
+        fork_height, _ = self.get_fork_point(summaries)
+        return True, fork_height
 
     async def validate_weight_proof(self, weight_proof: WeightProof) -> Tuple[bool, uint32, List[SubEpochSummary]]:
         assert self.blockchain is not None
-        assert len(weight_proof.sub_epochs) > 0
         if len(weight_proof.sub_epochs) == 0:
             return False, uint32(0), []
 
-        peak_height = weight_proof.recent_chain_data[-1].reward_chain_block.height
-        log.info(f"validate weight proof peak height {peak_height}")
-
-        # TODO: Consider if this can be spun off to a thread as an alternative to
-        #       sprinkling async sleeps around.  Also see the corresponding comment
-        #       in the wallet code.
-        #       all instances tagged as: 098faior2ru08d08ufa
-
         # timing reference: start
         summaries, sub_epoch_weight_list = _validate_sub_epoch_summaries(self.constants, weight_proof)
         await asyncio.sleep(0)  # break up otherwise multi-second sync code
         # timing reference: 1 second
-        if summaries is None:
+        if summaries is None or sub_epoch_weight_list is None:
             log.error("weight proof failed sub epoch data validation")
             return False, uint32(0), []
 
-        seed = summaries[-2].get_hash()
-        rng = random.Random(seed)
-        if not validate_sub_epoch_sampling(rng, sub_epoch_weight_list, weight_proof):
-            log.error("failed weight proof sub epoch sample validation")
-            return False, uint32(0), []
-
+        fork_point, ses_fork_idx = self.get_fork_point(summaries)
         # timing reference: 1 second
         # TODO: Consider implementing an async polling closer for the executor.
         with ProcessPoolExecutor(
@@ -634,66 +620,25 @@ class WeightProofHandler:
             # The shutdown file manager must be inside of the executor manager so that
             # we request the workers close prior to waiting for them to close.
             with _create_shutdown_file() as shutdown_file:
-                await asyncio.sleep(0)  # break up otherwise multi-second sync code
-                # timing reference: 1.1 second
-                constants, summary_bytes, wp_segment_bytes, wp_recent_chain_bytes = vars_to_bytes(
-                    self.constants, summaries, weight_proof
-                )
-                await asyncio.sleep(0)  # break up otherwise multi-second sync code
-
-                # timing reference: 2 second
-                recent_blocks_validation_task = asyncio.get_running_loop().run_in_executor(
-                    executor,
-                    _validate_recent_blocks,
-                    constants,
-                    wp_recent_chain_bytes,
-                    summary_bytes,
-                    pathlib.Path(shutdown_file.name),
-                )
-
-                # timing reference: 2 second
-                segments_validated, vdfs_to_validate = _validate_sub_epoch_segments(
-                    constants, rng, wp_segment_bytes, summary_bytes
-                )
-                await asyncio.sleep(0)  # break up otherwise multi-second sync code
-                if not segments_validated:
-                    return False, uint32(0), []
-
-                # timing reference: 4 second
-                vdf_chunks = chunks(vdfs_to_validate, self._num_processes)
-                vdf_tasks = []
-                # timing reference: 4 second
-                for chunk in vdf_chunks:
-                    byte_chunks = []
-                    for vdf_proof, classgroup, vdf_info in chunk:
-                        byte_chunks.append((bytes(vdf_proof), bytes(classgroup), bytes(vdf_info)))
-
-                    vdf_task = asyncio.get_running_loop().run_in_executor(
+                task: asyncio.Task = asyncio.create_task(
+                    validate_weight_proof_inner(
+                        self.constants,
                         executor,
-                        _validate_vdf_batch,
-                        constants,
-                        byte_chunks,
-                        pathlib.Path(shutdown_file.name),
+                        shutdown_file.name,
+                        self._num_processes,
+                        weight_proof,
+                        summaries,
+                        sub_epoch_weight_list,
+                        False,
+                        ses_fork_idx,
                     )
-                    vdf_tasks.append(vdf_task)
-                    # give other stuff a turn
-                    await asyncio.sleep(0)
+                )
 
-                # timing reference: 4 second
-                for vdf_task in asyncio.as_completed(fs=vdf_tasks):
-                    validated = await vdf_task
-                    if not validated:
-                        return False, uint32(0), []
+                valid, _ = await task
+        return valid, fork_point, summaries
 
-                valid_recent_blocks_task = recent_blocks_validation_task
-                valid_recent_blocks = await valid_recent_blocks_task
-        if not valid_recent_blocks:
-            log.error("failed validating weight proof recent blocks")
-            return False, uint32(0), []
-
-        return True, self.get_fork_point(summaries), summaries
-
-    def get_fork_point(self, received_summaries: List[SubEpochSummary]) -> uint32:
+    def get_fork_point(self, received_summaries: List[SubEpochSummary]) -> Tuple[uint32, int]:
+        # returns the fork height and ses index
         # iterate through sub epoch summaries to find fork point
         fork_point_index = 0
         ses_heights = self.blockchain.get_ses_heights()
@@ -707,14 +652,12 @@ class WeightProofHandler:
                 break
             fork_point_index = idx
 
-        if fork_point_index > 2:
+        if fork_point_index <= 2:
             # Two summeries can have different blocks and still be identical
             # This gets resolved after one full sub epoch
-            height = ses_heights[fork_point_index - 2]
-        else:
-            height = uint32(0)
+            return uint32(0), 0
 
-        return height
+        return ses_heights[fork_point_index - 2], fork_point_index
 
 
 def _get_weights_for_sampling(
@@ -995,12 +938,13 @@ def _validate_summaries_weight(constants: ConsensusConstants, sub_epoch_data_wei
 
 
 def _validate_sub_epoch_segments(
-    constants_dict: Dict,
+    constants: ConsensusConstants,
     rng: random.Random,
     weight_proof_bytes: bytes,
     summaries_bytes: List[bytes],
+    validate_from: int = 0,
 ):
-    constants, summaries = bytes_to_vars(constants_dict, summaries_bytes)
+    summaries = summaries_from_bytes(summaries_bytes)
     sub_epoch_segments: SubEpochSegments = SubEpochSegments.from_bytes(weight_proof_bytes)
     rc_sub_slot_hash = constants.GENESIS_CHALLENGE
     total_blocks, total_ip_iters = 0, 0
@@ -1023,6 +967,11 @@ def _validate_sub_epoch_segments(
         if not summaries[sub_epoch_n].reward_chain_hash == rc_sub_slot_hash:
             log.error(f"failed reward_chain_hash validation sub_epoch {sub_epoch_n}")
             return False
+
+        # skip validation up to fork height
+        if sub_epoch_n < validate_from:
+            continue
+
         for idx, segment in enumerate(segments):
             valid_segment, ip_iters, slot_iters, slots, vdf_list = _validate_segment(
                 constants, segment, curr_ssi, prev_ssi, curr_difficulty, prev_ses, idx == 0, sampled_seg_index == idx
@@ -1250,10 +1199,12 @@ def sub_slot_data_vdf_input(
 
 def validate_recent_blocks(
     constants: ConsensusConstants,
-    recent_chain: RecentChainData,
-    summaries: List[SubEpochSummary],
+    recent_chain_bytes: bytes,
+    summaries_bytes: List[bytes],
     shutdown_file_path: Optional[pathlib.Path] = None,
 ) -> Tuple[bool, List[bytes]]:
+    recent_chain: RecentChainData = RecentChainData.from_bytes(recent_chain_bytes)
+    summaries = summaries_from_bytes(summaries_bytes)
     sub_blocks = BlockCache({})
     first_ses_idx = _get_ses_idx(recent_chain.recent_chain_data)
     ses_idx = len(summaries) - len(first_ses_idx)
@@ -1339,39 +1290,6 @@ def validate_recent_blocks(
     return True, [bytes(sub) for sub in sub_blocks._block_records.values()]
 
 
-def _validate_recent_blocks(
-    constants_dict: Dict,
-    recent_chain_bytes: bytes,
-    summaries_bytes: List[bytes],
-    shutdown_file_path: Optional[pathlib.Path] = None,
-) -> bool:
-    constants, summaries = bytes_to_vars(constants_dict, summaries_bytes)
-    recent_chain: RecentChainData = RecentChainData.from_bytes(recent_chain_bytes)
-    success, records = validate_recent_blocks(
-        constants=constants,
-        recent_chain=recent_chain,
-        summaries=summaries,
-        shutdown_file_path=shutdown_file_path,
-    )
-    return success
-
-
-def _validate_recent_blocks_and_get_records(
-    constants_dict: Dict,
-    recent_chain_bytes: bytes,
-    summaries_bytes: List[bytes],
-    shutdown_file_path: Optional[pathlib.Path] = None,
-) -> Tuple[bool, List[bytes]]:
-    constants, summaries = bytes_to_vars(constants_dict, summaries_bytes)
-    recent_chain: RecentChainData = RecentChainData.from_bytes(recent_chain_bytes)
-    return validate_recent_blocks(
-        constants=constants,
-        recent_chain=recent_chain,
-        summaries=summaries,
-        shutdown_file_path=shutdown_file_path,
-    )
-
-
 def _validate_pospace_recent_chain(
     constants: ConsensusConstants,
     block: HeaderBlock,
@@ -1572,22 +1490,20 @@ def _get_curr_diff_ssi(constants: ConsensusConstants, idx, summaries):
     return curr_difficulty, curr_ssi
 
 
-def vars_to_bytes(constants: ConsensusConstants, summaries: List[SubEpochSummary], weight_proof: WeightProof):
-    constants_dict = recurse_jsonify(constants)
+def vars_to_bytes(summaries: List[SubEpochSummary], weight_proof: WeightProof):
     wp_recent_chain_bytes = bytes(RecentChainData(weight_proof.recent_chain_data))
     wp_segment_bytes = bytes(SubEpochSegments(weight_proof.sub_epoch_segments))
     summary_bytes = []
     for summary in summaries:
         summary_bytes.append(bytes(summary))
-    return constants_dict, summary_bytes, wp_segment_bytes, wp_recent_chain_bytes
+    return summary_bytes, wp_segment_bytes, wp_recent_chain_bytes
 
 
-def bytes_to_vars(constants_dict, summaries_bytes):
+def summaries_from_bytes(summaries_bytes: List[bytes]) -> List[SubEpochSummary]:
     summaries = []
     for summary in summaries_bytes:
         summaries.append(SubEpochSummary.from_bytes(summary))
-    constants: ConsensusConstants = dataclass_from_dict(ConsensusConstants, constants_dict)
-    return constants, summaries
+    return summaries
 
 
 def _get_last_ses_hash(
@@ -1660,7 +1576,7 @@ def blue_boxed_end_of_slot(sub_slot: EndOfSubSlotBundle):
 def validate_sub_epoch_sampling(rng, sub_epoch_weight_list, weight_proof):
     tip = weight_proof.recent_chain_data[-1]
     weight_to_check = _get_weights_for_sampling(rng, tip.weight, weight_proof.recent_chain_data)
-    sampled_sub_epochs: dict[int, bool] = {}
+    sampled_sub_epochs: Dict[int, bool] = {}
     for idx in range(1, len(sub_epoch_weight_list)):
         if _sample_sub_epoch(sub_epoch_weight_list[idx - 1], sub_epoch_weight_list[idx], weight_to_check):
             sampled_sub_epochs[idx - 1] = True
@@ -1719,9 +1635,10 @@ def validate_total_iters(
 
 
 def _validate_vdf_batch(
-    constants_dict, vdf_list: List[Tuple[bytes, bytes, bytes]], shutdown_file_path: Optional[pathlib.Path] = None
+    constants: ConsensusConstants,
+    vdf_list: List[Tuple[bytes, bytes, bytes]],
+    shutdown_file_path: Optional[pathlib.Path] = None,
 ):
-    constants: ConsensusConstants = dataclass_from_dict(ConsensusConstants, constants_dict)
 
     for vdf_proof_bytes, class_group_bytes, info in vdf_list:
         vdf = VDFProof.from_bytes(vdf_proof_bytes)
@@ -1735,3 +1652,77 @@ def _validate_vdf_batch(
             return False
 
     return True
+
+
+async def validate_weight_proof_inner(
+    constants,
+    executor,
+    shutdown_file_name,
+    num_processes,
+    weight_proof: WeightProof,
+    summaries: List[SubEpochSummary],
+    sub_epoch_weight_list: List[uint128],
+    skip_segment_validation: bool,
+    validate_from: int,
+) -> Tuple[bool, List[BlockRecord]]:
+    assert len(weight_proof.sub_epochs) > 0
+    if len(weight_proof.sub_epochs) == 0:
+        return False, []
+
+    peak_height = weight_proof.recent_chain_data[-1].reward_chain_block.height
+    log.info(f"validate weight proof peak height {peak_height}")
+    seed = summaries[-2].get_hash()
+    rng = random.Random(seed)
+    if not validate_sub_epoch_sampling(rng, sub_epoch_weight_list, weight_proof):
+        log.error("failed weight proof sub epoch sample validation")
+        return False, []
+
+    summary_bytes, wp_segment_bytes, wp_recent_chain_bytes = vars_to_bytes(summaries, weight_proof)
+    recent_blocks_validation_task = executor.submit(
+        validate_recent_blocks,
+        constants,
+        wp_recent_chain_bytes,
+        summary_bytes,
+        pathlib.Path(shutdown_file_name),
+    )
+
+    if not skip_segment_validation:
+        segments_validated, vdfs_to_validate = _validate_sub_epoch_segments(
+            constants, rng, wp_segment_bytes, summary_bytes, validate_from
+        )
+        await asyncio.sleep(0)  # break up otherwise multi-second sync code
+
+        if not segments_validated:
+            return False, []
+
+        vdf_chunks = chunks(vdfs_to_validate, num_processes)
+        vdf_tasks: List[Awaitable] = []
+        for chunk in vdf_chunks:
+            byte_chunks = []
+            for vdf_proof, classgroup, vdf_info in chunk:
+                byte_chunks.append((bytes(vdf_proof), bytes(classgroup), bytes(vdf_info)))
+            vdf_task = asyncio.get_running_loop().run_in_executor(
+                executor,
+                _validate_vdf_batch,
+                constants,
+                byte_chunks,
+                pathlib.Path(shutdown_file_name),
+            )
+            vdf_tasks.append(vdf_task)
+            # give other stuff a turn
+            await asyncio.sleep(0)
+
+        for vdf_task in asyncio.as_completed(fs=vdf_tasks):
+            validated = await vdf_task
+            if not validated:
+                return False, []
+
+    valid_recent_blocks, records_bytes = recent_blocks_validation_task.result()
+
+    if not valid_recent_blocks or records_bytes is None:
+        log.error("failed validating weight proof recent blocks")
+        # Verify the data
+        return False, []
+
+    records = [BlockRecord.from_bytes(b) for b in records_bytes]
+    return True, records
