@@ -68,7 +68,7 @@ class PlotManager:
         self._lock = threading.Lock()
         self._refresh_thread = None
         self._refreshing_enabled = False
-        self._refresh_callback = refresh_callback  # type: ignore
+        self._refresh_callback = refresh_callback
         self._initial = True
 
     def __enter__(self):
@@ -87,7 +87,7 @@ class PlotManager:
             self._initial = True
 
     def set_refresh_callback(self, callback: Callable):
-        self._refresh_callback = callback  # type: ignore
+        self._refresh_callback = callback
 
     def set_public_keys(self, farmer_public_keys: List[G1Element], pool_public_keys: List[G1Element]):
         self.farmer_public_keys = farmer_public_keys
@@ -99,11 +99,11 @@ class PlotManager:
     def public_keys_available(self):
         return len(self.farmer_public_keys) and len(self.pool_public_keys)
 
-    def plot_count(self):
+    def plot_count(self) -> int:
         with self:
             return len(self.plots)
 
-    def get_duplicates(self):
+    def get_duplicates(self) -> List[Path]:
         result = []
         for plot_filename, paths_entry in self.plot_filename_paths.items():
             _, duplicated_paths = paths_entry
@@ -121,13 +121,13 @@ class PlotManager:
             self._refresh_thread = threading.Thread(target=self._refresh_task, args=(sleep_interval_ms,))
             self._refresh_thread.start()
 
-    def stop_refreshing(self):
+    def stop_refreshing(self) -> None:
         self._refreshing_enabled = False
         if self._refresh_thread is not None and self._refresh_thread.is_alive():
             self._refresh_thread.join()
             self._refresh_thread = None
 
-    def trigger_refresh(self):
+    def trigger_refresh(self) -> None:
         log.debug("trigger_refresh")
         self.last_refresh_time = 0
 
@@ -184,7 +184,7 @@ class PlotManager:
                 for filename in filenames_to_remove:
                     del self.plot_filename_paths[filename]
 
-                for remaining, batch in list_to_batches(list(plot_paths), self.refresh_parameter.batch_size):
+                for remaining, batch in list_to_batches(sorted(list(plot_paths)), self.refresh_parameter.batch_size):
                     batch_result: PlotRefreshResult = self.refresh_batch(batch, plot_directories)
                     if not self._refreshing_enabled:
                         self.log.debug("refresh_plots: Aborted")
