@@ -1,9 +1,14 @@
-from typing import Any, Callable, Dict, List, Optional
+from typing import Any, Dict, List, Optional
+
+from clvm.casts import int_from_bytes
 
 from chinilla.consensus.block_record import BlockRecord
 from chinilla.consensus.pos_quality import UI_ACTUAL_SPACE_CONSTANT_FACTOR
 from chinilla.full_node.full_node import FullNode
+from chinilla.full_node.generator import setup_generator_args
 from chinilla.full_node.mempool_check_conditions import get_puzzle_and_solution_for_coin
+from chinilla.rpc.rpc_server import Endpoint, EndpointResult
+from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.program import Program, SerializedProgram
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.coin_record import CoinRecord
@@ -17,6 +22,7 @@ from chinilla.util.byte_types import hexstr_to_bytes
 from chinilla.util.ints import uint32, uint64, uint128
 from chinilla.util.log_exceptions import log_exceptions
 from chinilla.util.ws_message import WsRpcMessage, create_payload_dict
+from chinilla.wallet.puzzles.decompress_block_spends import DECOMPRESS_BLOCK_SPENDS
 
 
 def coin_record_dict_backwards_compat(coin_record: Dict[str, Any]):
@@ -30,7 +36,7 @@ class FullNodeRpcApi:
         self.service_name = "chinilla_full_node"
         self.cached_blockchain_state: Optional[Dict] = None
 
-    def get_routes(self) -> Dict[str, Callable]:
+    def get_routes(self) -> Dict[str, Endpoint]:
         return {
             # Blockchain
             "/get_blockchain_state": self.get_blockchain_state,
@@ -40,6 +46,7 @@ class FullNodeRpcApi:
             "/get_block_record_by_height": self.get_block_record_by_height,
             "/get_block_record": self.get_block_record,
             "/get_block_records": self.get_block_records,
+            "/get_block_spends": self.get_block_spends,
             "/get_unfinished_block_headers": self.get_unfinished_block_headers,
             "/get_network_space": self.get_network_space,
             "/get_additions_and_removals": self.get_additions_and_removals,
@@ -95,11 +102,11 @@ class FullNodeRpcApi:
 
     # this function is just here for backwards-compatibility. It will probably
     # be removed in the future
-    async def get_initial_freeze_period(self, _: Dict):
+    async def get_initial_freeze_period(self, _: Dict) -> EndpointResult:
         # Mon May 03 2021 17:00:00 GMT+0000
         return {"INITIAL_FREEZE_END_TIMESTAMP": 1620061200}
 
-    async def get_blockchain_state(self, _request: Dict):
+    async def get_blockchain_state(self, _request: Dict) -> EndpointResult:
         """
         Returns a summary of the node's view of the blockchain.
         """
@@ -179,7 +186,9 @@ class FullNodeRpcApi:
             mempool_min_fee_5m = 0
             mempool_max_total_cost = 0
         if self.service.server is not None:
-            is_connected = len(self.service.server.get_full_node_connections()) > 0
+            is_connected = len(self.service.server.get_full_node_connections()) > 0 or "simulator" in str(
+                self.service.config.get("selected_network")
+            )
         else:
             is_connected = False
         synced = await self.service.synced() and is_connected
@@ -213,12 +222,12 @@ class FullNodeRpcApi:
         self.cached_blockchain_state = dict(response["blockchain_state"])
         return response
 
-    async def get_network_info(self, request: Dict):
+    async def get_network_info(self, request: Dict) -> EndpointResult:
         network_name = self.service.config["selected_network"]
         address_prefix = self.service.config["network_overrides"]["config"][network_name]["address_prefix"]
         return {"network_name": network_name, "network_prefix": address_prefix}
 
-    async def get_recent_signage_point_or_eos(self, request: Dict):
+    async def get_recent_signage_point_or_eos(self, request: Dict) -> EndpointResult:
         if "sp_hash" not in request:
             challenge_hash: bytes32 = bytes32.from_hexstr(request["challenge_hash"])
             # This is the case of getting an end of slot
@@ -307,7 +316,7 @@ class FullNodeRpcApi:
 
         return {"signage_point": sp, "time_received": time_received, "reverted": True}
 
-    async def get_block(self, request: Dict) -> Optional[Dict]:
+    async def get_block(self, request: Dict) -> EndpointResult:
         if "header_hash" not in request:
             raise ValueError("No header_hash in request")
         header_hash = bytes32.from_hexstr(request["header_hash"])
@@ -318,7 +327,7 @@ class FullNodeRpcApi:
 
         return {"block": block}
 
-    async def get_blocks(self, request: Dict) -> Optional[Dict]:
+    async def get_blocks(self, request: Dict) -> EndpointResult:
         if "start" not in request:
             raise ValueError("No start in request")
         if "end" not in request:
@@ -348,7 +357,7 @@ class FullNodeRpcApi:
             json_blocks.append(json)
         return {"blocks": json_blocks}
 
-    async def get_block_count_metrics(self, request: Dict):
+    async def get_block_count_metrics(self, request: Dict) -> EndpointResult:
         compact_blocks = 0
         uncompact_blocks = 0
         with log_exceptions(self.service.log, consume=True):
@@ -368,7 +377,7 @@ class FullNodeRpcApi:
             }
         }
 
-    async def get_block_records(self, request: Dict) -> Optional[Dict]:
+    async def get_block_records(self, request: Dict) -> EndpointResult:
         if "start" not in request:
             raise ValueError("No start in request")
         if "end" not in request:
@@ -398,7 +407,33 @@ class FullNodeRpcApi:
             records.append(record)
         return {"block_records": records}
 
-    async def get_block_record_by_height(self, request: Dict) -> Optional[Dict]:
+    async def get_block_spends(self, request: Dict) -> EndpointResult:
+        if "header_hash" not in request:
+            raise ValueError("No header_hash in request")
+        header_hash = bytes32.from_hexstr(request["header_hash"])
+        full_block: Optional[FullBlock] = await self.service.block_store.get_full_block(header_hash)
+        if full_block is None:
+            raise ValueError(f"Block {header_hash.hex()} not found")
+
+        spends: List[CoinSpend] = []
+        block_generator = await self.service.blockchain.get_block_generator(full_block)
+        if block_generator is None:  # if block is not a transaction block.
+            return {"block_spends": spends}
+
+        block_program, block_program_args = setup_generator_args(block_generator)
+        _, coin_spends = DECOMPRESS_BLOCK_SPENDS.run_with_cost(
+            self.service.constants.MAX_BLOCK_COST_CLVM, block_program, block_program_args
+        )
+
+        for spend in coin_spends.as_iter():
+            parent, puzzle, amount, solution = spend.as_iter()
+            puzzle_hash = puzzle.get_tree_hash()
+            coin = Coin(parent.atom, puzzle_hash, int_from_bytes(amount.atom))
+            spends.append(CoinSpend(coin, puzzle, solution))
+
+        return {"block_spends": spends}
+
+    async def get_block_record_by_height(self, request: Dict) -> EndpointResult:
         if "height" not in request:
             raise ValueError("No height in request")
         height = request["height"]
@@ -417,7 +452,7 @@ class FullNodeRpcApi:
             raise ValueError(f"Block {header_hash} does not exist")
         return {"block_record": record}
 
-    async def get_block_record(self, request: Dict):
+    async def get_block_record(self, request: Dict) -> EndpointResult:
         if "header_hash" not in request:
             raise ValueError("header_hash not in request")
         header_hash_str = request["header_hash"]
@@ -431,7 +466,7 @@ class FullNodeRpcApi:
 
         return {"block_record": record}
 
-    async def get_unfinished_block_headers(self, request: Dict) -> Optional[Dict]:
+    async def get_unfinished_block_headers(self, request: Dict) -> EndpointResult:
 
         peak: Optional[BlockRecord] = self.service.blockchain.get_peak()
         if peak is None:
@@ -452,7 +487,7 @@ class FullNodeRpcApi:
                 response_headers.append(unfinished_header_block)
         return {"headers": response_headers}
 
-    async def get_network_space(self, request: Dict) -> Optional[Dict]:
+    async def get_network_space(self, request: Dict) -> EndpointResult:
         """
         Retrieves an estimate of total space validating the chain
         between two block header hashes.
@@ -492,7 +527,7 @@ class FullNodeRpcApi:
         )
         return {"space": uint128(int(network_space_bytes_estimate))}
 
-    async def get_coin_records_by_puzzle_hash(self, request: Dict) -> Optional[Dict]:
+    async def get_coin_records_by_puzzle_hash(self, request: Dict) -> EndpointResult:
         """
         Retrieves the coins for a given puzzlehash, by default returns unspent coins.
         """
@@ -511,7 +546,7 @@ class FullNodeRpcApi:
 
         return {"coin_records": [coin_record_dict_backwards_compat(cr.to_json_dict()) for cr in coin_records]}
 
-    async def get_coin_records_by_puzzle_hashes(self, request: Dict) -> Optional[Dict]:
+    async def get_coin_records_by_puzzle_hashes(self, request: Dict) -> EndpointResult:
         """
         Retrieves the coins for a given puzzlehash, by default returns unspent coins.
         """
@@ -533,7 +568,7 @@ class FullNodeRpcApi:
 
         return {"coin_records": [coin_record_dict_backwards_compat(cr.to_json_dict()) for cr in coin_records]}
 
-    async def get_coin_record_by_name(self, request: Dict) -> Optional[Dict]:
+    async def get_coin_record_by_name(self, request: Dict) -> EndpointResult:
         """
         Retrieves a coin record by it's name.
         """
@@ -547,7 +582,7 @@ class FullNodeRpcApi:
 
         return {"coin_record": coin_record_dict_backwards_compat(coin_record.to_json_dict())}
 
-    async def get_coin_records_by_names(self, request: Dict) -> Optional[Dict]:
+    async def get_coin_records_by_names(self, request: Dict) -> EndpointResult:
         """
         Retrieves the coins for given coin IDs, by default returns unspent coins.
         """
@@ -569,7 +604,7 @@ class FullNodeRpcApi:
 
         return {"coin_records": [coin_record_dict_backwards_compat(cr.to_json_dict()) for cr in coin_records]}
 
-    async def get_coin_records_by_parent_ids(self, request: Dict) -> Optional[Dict]:
+    async def get_coin_records_by_parent_ids(self, request: Dict) -> EndpointResult:
         """
         Retrieves the coins for given parent coin IDs, by default returns unspent coins.
         """
@@ -591,7 +626,7 @@ class FullNodeRpcApi:
 
         return {"coin_records": [coin_record_dict_backwards_compat(cr.to_json_dict()) for cr in coin_records]}
 
-    async def get_coin_records_by_hint(self, request: Dict) -> Optional[Dict]:
+    async def get_coin_records_by_hint(self, request: Dict) -> EndpointResult:
         """
         Retrieves coins by hint, by default returns unspent coins.
         """
@@ -620,7 +655,7 @@ class FullNodeRpcApi:
 
         return {"coin_records": [coin_record_dict_backwards_compat(cr.to_json_dict()) for cr in coin_records]}
 
-    async def push_tx(self, request: Dict) -> Optional[Dict]:
+    async def push_tx(self, request: Dict) -> EndpointResult:
         if "spend_bundle" not in request:
             raise ValueError("Spend bundle not in request")
 
@@ -645,7 +680,7 @@ class FullNodeRpcApi:
             "status": status.name,
         }
 
-    async def get_puzzle_and_solution(self, request: Dict) -> Optional[Dict]:
+    async def get_puzzle_and_solution(self, request: Dict) -> EndpointResult:
         coin_name: bytes32 = bytes32.from_hexstr(request["coin_id"])
         height = request["height"]
         coin_record = await self.service.coin_store.get_coin_record(coin_name)
@@ -671,7 +706,7 @@ class FullNodeRpcApi:
         solution_ser: SerializedProgram = SerializedProgram.from_program(Program.to(solution))
         return {"coin_solution": CoinSpend(coin_record.coin, puzzle_ser, solution_ser)}
 
-    async def get_additions_and_removals(self, request: Dict) -> Optional[Dict]:
+    async def get_additions_and_removals(self, request: Dict) -> EndpointResult:
         if "header_hash" not in request:
             raise ValueError("No header_hash in request")
         header_hash = bytes32.from_hexstr(request["header_hash"])
@@ -691,17 +726,17 @@ class FullNodeRpcApi:
             "removals": [coin_record_dict_backwards_compat(cr.to_json_dict()) for cr in removals],
         }
 
-    async def get_all_mempool_tx_ids(self, request: Dict) -> Optional[Dict]:
+    async def get_all_mempool_tx_ids(self, request: Dict) -> EndpointResult:
         ids = list(self.service.mempool_manager.mempool.spends.keys())
         return {"tx_ids": ids}
 
-    async def get_all_mempool_items(self, request: Dict) -> Optional[Dict]:
+    async def get_all_mempool_items(self, request: Dict) -> EndpointResult:
         spends = {}
         for tx_id, item in self.service.mempool_manager.mempool.spends.items():
             spends[tx_id.hex()] = item
         return {"mempool_items": spends}
 
-    async def get_mempool_item_by_tx_id(self, request: Dict) -> Optional[Dict]:
+    async def get_mempool_item_by_tx_id(self, request: Dict) -> EndpointResult:
         if "tx_id" not in request:
             raise ValueError("No tx_id in request")
         tx_id: bytes32 = bytes32.from_hexstr(request["tx_id"])
