@@ -110,20 +110,20 @@ class ChinillaServer:
         network_id: str,
         inbound_rate_limit_percent: int,
         outbound_rate_limit_percent: int,
+        capabilities: List[Tuple[uint16, str]],
         root_path: Path,
         config: Dict,
         private_ca_crt_key: Tuple[Path, Path],
         chinilla_ca_crt_key: Tuple[Path, Path],
         name: str = None,
-        introducer_peers: Optional[IntroducerPeers] = None,
     ):
         # Keeps track of all connections to and from this node.
-        logging.basicConfig(level=logging.DEBUG)
         self.all_connections: Dict[bytes32, WSChinillaConnection] = {}
         self.tasks: Set[asyncio.Task] = set()
 
         self.connection_by_type: Dict[NodeType, Dict[bytes32, WSChinillaConnection]] = {
             NodeType.FULL_NODE: {},
+            NodeType.DATA_LAYER: {},
             NodeType.WALLET: {},
             NodeType.HARVESTER: {},
             NodeType.FARMER: {},
@@ -133,7 +133,7 @@ class ChinillaServer:
 
         self._port = port  # TCP port to identify our node
         self._local_type: NodeType = local_type
-
+        self._local_capabilities_for_handshake = capabilities
         self._ping_interval = ping_interval
         self._network_id = network_id
         self._inbound_rate_limit_percent = inbound_rate_limit_percent
@@ -143,6 +143,7 @@ class ChinillaServer:
         self._tasks: List[asyncio.Task] = []
 
         self.log = logging.getLogger(name if name else __name__)
+        self.log.info("Service capabilities: %s", self._local_capabilities_for_handshake)
 
         # Our unique random node id that we will send to other peers, regenerated on launch
         self.api = api
@@ -312,14 +313,10 @@ class ChinillaServer:
                 peer_id,
                 self._inbound_rate_limit_percent,
                 self._outbound_rate_limit_percent,
+                self._local_capabilities_for_handshake,
                 close_event,
             )
-            await connection.perform_handshake(
-                self._network_id,
-                protocol_version,
-                self._port,
-                self._local_type,
-            )
+            await connection.perform_handshake(self._network_id, protocol_version, self._port, self._local_type)
 
             # Limit inbound connections to config's specifications.
             if not self.accept_inbound_connections(connection.connection_type) and not is_in_network(
@@ -465,14 +462,10 @@ class ChinillaServer:
                 peer_id,
                 self._inbound_rate_limit_percent,
                 self._outbound_rate_limit_percent,
+                self._local_capabilities_for_handshake,
                 session=session,
             )
-            await connection.perform_handshake(
-                self._network_id,
-                protocol_version,
-                self._port,
-                self._local_type,
-            )
+            await connection.perform_handshake(self._network_id, protocol_version, self._port, self._local_type)
             await self.connection_added(connection, on_connect)
             # the session has been adopted by the connection, don't close it at
             # the end of the function
