@@ -1,4 +1,5 @@
 from dataclasses import dataclass
+from typing import Optional
 
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.ints import uint32
@@ -17,3 +18,10 @@ class ReorgProtocol(Streamable):
     old_index: uint32
     new_index: uint32
     puzzle_hash: bytes32
+    seed: Optional[bytes32]
+
+
+@streamable
+@dataclass(frozen=True)
+class GetAllCoinsProtocol(Streamable):
+    include_spent_coins: bool
