@@ -13,10 +13,9 @@ from typing import Any, Callable, Dict, Iterator, Optional, Union
 
 import pkg_resources
 import yaml
-from filelock import FileLock
 from typing_extensions import Literal
 
-from chinilla.util.path import mkdir
+from chinilla.util.lock import Lockfile
 
 PEER_DB_PATH_KEY_DEPRECATED = "peer_db_path"  # replaced by "peers_file_path"
 WALLET_PEERS_PATH_KEY_DEPRECATED = "wallet_peers_path"  # replaced by "wallet_peers_file_path"
@@ -33,7 +32,7 @@ def create_default_chinilla_config(root_path: Path, filenames=["config.yaml"]) -
         default_config_file_data: str = initial_config_file(filename)
         path: Path = config_path_for_filename(root_path, filename)
         tmp_path: Path = path.with_suffix("." + str(os.getpid()))
-        mkdir(path.parent)
+        path.parent.mkdir(parents=True, exist_ok=True)
         with open(tmp_path, "w") as f:
             f.write(default_config_file_data)
         try:
@@ -55,15 +54,23 @@ def lock_config(root_path: Path, filename: Union[str, Path]) -> Iterator[None]:
     #       configuration file without having loaded it right there.  This usage
     #       should probably be removed and this function made private.
     config_path = config_path_for_filename(root_path, filename)
-    lock_path: Path = config_path.with_name(config_path.name + ".lock")
-    with FileLock(lock_path):
+    with Lockfile.create(config_path):
         yield
 
 
 @contextlib.contextmanager
-def lock_and_load_config(root_path: Path, filename: Union[str, Path]) -> Iterator[Dict[str, Any]]:
+def lock_and_load_config(
+    root_path: Path,
+    filename: Union[str, Path],
+    fill_missing_services: bool = False,
+) -> Iterator[Dict[str, Any]]:
     with lock_config(root_path=root_path, filename=filename):
-        config = _load_config_maybe_locked(root_path=root_path, filename=filename, acquire_lock=False)
+        config = _load_config_maybe_locked(
+            root_path=root_path,
+            filename=filename,
+            acquire_lock=False,
+            fill_missing_services=fill_missing_services,
+        )
         yield config
 
 
@@ -85,6 +92,7 @@ def load_config(
     filename: Union[str, Path],
     sub_config: Optional[str] = None,
     exit_on_error: bool = True,
+    fill_missing_services: bool = False,
 ) -> Dict:
     return _load_config_maybe_locked(
         root_path=root_path,
@@ -92,6 +100,7 @@ def load_config(
         sub_config=sub_config,
         exit_on_error=exit_on_error,
         acquire_lock=True,
+        fill_missing_services=fill_missing_services,
     )
 
 
@@ -101,6 +110,7 @@ def _load_config_maybe_locked(
     sub_config: Optional[str] = None,
     exit_on_error: bool = True,
     acquire_lock: bool = True,
+    fill_missing_services: bool = False,
 ) -> Dict:
     # This must be called under an acquired config lock, or acquire_lock should be True
 
@@ -125,6 +135,8 @@ def _load_config_maybe_locked(
                 log.error(f"yaml.safe_load returned None: {path}")
                 time.sleep(i * 0.1)
                 continue
+            if fill_missing_services:
+                r.update(load_defaults_for_missing_services(config=r, config_name=path.name))
             if sub_config is not None:
                 r = r.get(sub_config)
             return r
@@ -135,14 +147,19 @@ def _load_config_maybe_locked(
     raise RuntimeError("Was not able to read config file successfully")
 
 
-def load_config_cli(root_path: Path, filename: str, sub_config: Optional[str] = None) -> Dict:
+def load_config_cli(
+    root_path: Path,
+    filename: str,
+    sub_config: Optional[str] = None,
+    fill_missing_services: bool = False,
+) -> Dict:
     """
     Loads configuration from the specified filename, in the config directory,
     and then overrides any properties using the passed in command line arguments.
     Nested properties in the config file can be used in the command line with ".",
     for example --farmer_peer.host. Does not support lists.
     """
-    config = load_config(root_path, filename, sub_config)
+    config = load_config(root_path, filename, sub_config, fill_missing_services=fill_missing_services)
 
     flattened_props = flatten_properties(config)
     parser = argparse.ArgumentParser()
@@ -182,13 +199,16 @@ def unflatten_properties(config: Dict) -> Dict:
 
 
 def add_property(d: Dict, partial_key: str, value: Any):
-    key_1, key_2 = partial_key.split(".", maxsplit=1)
-    if key_1 not in d:
-        d[key_1] = {}
-    if "." in key_2:
-        add_property(d[key_1], key_2, value)
+    if "." not in partial_key:  # root of dict
+        d[partial_key] = value
     else:
-        d[key_1][key_2] = value
+        key_1, key_2 = partial_key.split(".", maxsplit=1)
+        if key_1 not in d:
+            d[key_1] = {}
+        if "." in key_2:
+            add_property(d[key_1], key_2, value)
+        else:
+            d[key_1][key_2] = value
 
 
 def str2bool(v: Union[str, bool]) -> bool:
@@ -272,3 +292,34 @@ def override_config(config: Dict[str, Any], config_overrides: Optional[Dict[str,
     for k, v in config_overrides.items():
         add_property(new_config, k, v)
     return new_config
+
+
+def selected_network_address_prefix(config: Dict[str, Any]) -> str:
+    address_prefix = config["network_overrides"]["config"][config["selected_network"]]["address_prefix"]
+    return address_prefix
+
+
+def load_defaults_for_missing_services(config: Dict[str, Any], config_name: str) -> Dict[str, Any]:
+    services = ["data_layer"]
+    missing_services = [service for service in services if service not in config]
+    defaulted = {}
+    if len(missing_services) > 0:
+        marshalled_default_config: str = initial_config_file(config_name)
+
+        unmarshalled_default_config = yaml.safe_load(marshalled_default_config)
+
+        for service in missing_services:
+            defaulted[service] = unmarshalled_default_config[service]
+
+            if "logging" in defaulted[service]:
+                to_be_referenced = config.get("logging")
+                if to_be_referenced is not None:
+                    defaulted[service]["logging"] = to_be_referenced
+
+            if "selected_network" in defaulted[service]:
+                to_be_referenced = config.get("selected_network")
+                if to_be_referenced is not None:
+                    # joining to hopefully force a new string of the same value
+                    defaulted[service]["selected_network"] = "".join(to_be_referenced)
+
+    return defaulted
