@@ -10,13 +10,13 @@ from chinilla.wallet.wallet_coin_record import WalletCoinRecord
 
 async def select_coins(
     spendable_amount: uint128,
-    max_coin_amount: int,
+    max_coin_amount: uint64,
     spendable_coins: List[WalletCoinRecord],
     unconfirmed_removals: Dict[bytes32, Coin],
     log: logging.Logger,
     amount: uint128,
     exclude: Optional[List[Coin]] = None,
-    min_coin_amount: Optional[uint128] = None,
+    min_coin_amount: Optional[uint64] = None,
 ) -> Set[Coin]:
     """
     Returns a set of coins that can be used for generating a new transaction.
@@ -24,7 +24,7 @@ async def select_coins(
     if exclude is None:
         exclude = []
     if min_coin_amount is None:
-        min_coin_amount = uint128(0)
+        min_coin_amount = uint64(0)
 
     if amount > spendable_amount:
         error_msg = (
@@ -44,7 +44,7 @@ async def select_coins(
             continue
         if coin_record.coin in exclude:
             continue
-        if coin_record.coin.amount < min_coin_amount:
+        if coin_record.coin.amount < min_coin_amount or coin_record.coin.amount > max_coin_amount:
             continue
         valid_spendable_coins.append(coin_record.coin)
         sum_spendable_coins += coin_record.coin.amount
@@ -56,6 +56,11 @@ async def select_coins(
             f"Transaction for {amount} is greater than spendable balance of {sum_spendable_coins}. "
             "There may be other transactions pending or our minimum coin amount is too high."
         )
+    if amount == 0 and sum_spendable_coins == 0:
+        raise ValueError(
+            "No coins available to spend, you can not create a coin with an amount of 0,"
+            " without already having coins."
+        )
 
     # Sort the coins by amount
     valid_spendable_coins.sort(reverse=True, key=lambda r: r.amount)
@@ -74,7 +79,7 @@ async def select_coins(
         if coin.amount < amount:
             smaller_coin_sum += coin.amount
             smaller_coins.append(coin)
-    if smaller_coin_sum == amount and len(smaller_coins) < max_num_coins:
+    if smaller_coin_sum == amount and len(smaller_coins) < max_num_coins and amount != 0:
         log.debug(f"Selected all smaller coins because they equate to an exact match of the target.: {smaller_coins}")
         return set(smaller_coins)
     elif smaller_coin_sum < amount:
@@ -91,13 +96,13 @@ async def select_coins(
                 greater_coin = select_smallest_coin_over_target(amount, valid_spendable_coins)
                 if greater_coin is None:
                     raise ValueError(
-                        f"Transaction of {amount} mojo would use more than "
+                        f"Transaction of {amount} vojo would use more than "
                         f"{max_num_coins} coins. Try sending a smaller amount"
                     )
                 coin_set = {greater_coin}
         return coin_set
     else:
-        # if smaller_coin_sum == amount and len(smaller_coins) >= max_num_coins.
+        # if smaller_coin_sum == amount and (len(smaller_coins) >= max_num_coins or amount == 0)
         potential_large_coin: Optional[Coin] = select_smallest_coin_over_target(amount, valid_spendable_coins)
         if potential_large_coin is None:
             raise ValueError("Too many coins are required to make this transaction")
