@@ -22,4 +22,4 @@ class WalletCoinRecord:
     wallet_id: int
 
     def name(self) -> bytes32:
-        return self.coin.name()
+        return self.coin.name()  # type: ignore
