@@ -9,7 +9,7 @@ import tempfile
 
 from tests.setup_nodes import setup_node_and_wallet, setup_n_nodes, setup_two_nodes
 from pathlib import Path
-from typing import AsyncIterator, List, Tuple
+from typing import Any, AsyncIterator, Dict, List, Tuple
 from chinilla.server.start_service import Service
 
 # Set spawn after stdlib imports, but before other imports
@@ -18,9 +18,9 @@ from chinilla.protocols import full_node_protocol
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
+from chinilla.util.config import create_default_chinilla_config, lock_and_load_config
 from chinilla.util.ints import uint16
 from tests.core.node_height import node_height_at_least
-from tests.pools.test_pool_rpc import wallet_is_synced
 from tests.setup_nodes import (
     setup_simulators_and_wallets,
     setup_node_and_wallet,
@@ -32,14 +32,15 @@ from tests.setup_nodes import (
     setup_two_nodes,
 )
 from tests.simulation.test_simulation import test_constants_modified
-from tests.time_out_assert import time_out_assert
-from tests.wallet_tools import WalletTool
+from chinilla.simulator.time_out_assert import time_out_assert
+from chinilla.simulator.wallet_tools import WalletTool
+from tests.util.wallet_is_synced import wallet_is_synced
 
 multiprocessing.set_start_method("spawn")
 
 from pathlib import Path
 from chinilla.util.keyring_wrapper import KeyringWrapper
-from tests.block_tools import BlockTools, test_constants, create_block_tools, create_block_tools_async
+from chinilla.simulator.block_tools import BlockTools, test_constants, create_block_tools, create_block_tools_async
 from tests.util.keyring import TempKeyring
 from tests.setup_nodes import setup_farmer_multi_harvester
 
@@ -62,7 +63,7 @@ def block_tools_fixture(get_keychain) -> BlockTools:
 # to run the tests, change the `self_hostname` fixture
 @pytest_asyncio.fixture(scope="session")
 def self_hostname():
-    return "localhost"
+    return "127.0.0.1"
 
 
 # NOTE:
@@ -95,11 +96,6 @@ def db_version(request):
     return request.param
 
 
-@pytest.fixture(scope="function", params=[1000000, 2300000])
-def softfork_height(request):
-    return request.param
-
-
 saved_blocks_version = "rc5"
 
 
@@ -216,11 +212,23 @@ if os.getenv("_PYTEST_RAISE", "0") != "0":
 
 
 @pytest_asyncio.fixture(scope="function")
-async def wallet_node(self_hostname):
-    async for _ in setup_node_and_wallet(test_constants, self_hostname):
+async def wallet_node(self_hostname, request):
+    params = {}
+    if request and request.param_index > 0:
+        params = request.param
+    async for _ in setup_node_and_wallet(test_constants, self_hostname, **params):
         yield _
 
 
+@pytest_asyncio.fixture(scope="function")
+async def node_with_params(request):
+    params = {}
+    if request:
+        params = request.param
+    async for (sims, wallets, bt) in setup_simulators_and_wallets(1, 0, {}, **params):
+        yield sims[0]
+
+
 @pytest_asyncio.fixture(scope="function")
 async def two_nodes(db_version, self_hostname):
     async for _ in setup_two_nodes(test_constants, db_version=db_version, self_hostname=self_hostname):
@@ -252,16 +260,16 @@ async def five_nodes(db_version, self_hostname):
 
 
 @pytest_asyncio.fixture(scope="function")
-async def wallet_nodes(bt):
+async def wallet_nodes():
     async_gen = setup_simulators_and_wallets(2, 1, {"MEMPOOL_BLOCK_BUFFER": 1, "MAX_BLOCK_COST_CLVM": 400000000})
-    nodes, wallets = await async_gen.__anext__()
+    nodes, wallets, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     full_node_2 = nodes[1]
     server_1 = full_node_1.full_node.server
     server_2 = full_node_2.full_node.server
     wallet_a = bt.get_pool_wallet_tool()
     wallet_receiver = WalletTool(full_node_1.full_node.constants)
-    yield full_node_1, full_node_2, server_1, server_2, wallet_a, wallet_receiver
+    yield full_node_1, full_node_2, server_1, server_2, wallet_a, wallet_receiver, bt
 
     async for _ in async_gen:
         yield _
@@ -292,8 +300,17 @@ async def wallet_node_100_pk():
 
 
 @pytest_asyncio.fixture(scope="function")
-async def two_wallet_nodes():
-    async for _ in setup_simulators_and_wallets(1, 2, {}):
+async def two_wallet_nodes(request):
+    params = {}
+    if request and request.param_index > 0:
+        params = request.param
+    async for _ in setup_simulators_and_wallets(1, 2, {}, **params):
+        yield _
+
+
+@pytest_asyncio.fixture(scope="function")
+async def two_wallet_nodes_custom_spam_filtering(spam_filter_after_n_txs, hcx_spam_amount):
+    async for _ in setup_simulators_and_wallets(1, 2, {}, spam_filter_after_n_txs, hcx_spam_amount):
         yield _
 
 
@@ -349,36 +366,30 @@ async def wallet_nodes_mempool_perf(bt):
 
 
 @pytest_asyncio.fixture(scope="module")
-async def wallet_nodes_perf(bt):
+async def wallet_nodes_perf():
     async_gen = setup_simulators_and_wallets(1, 1, {"MEMPOOL_BLOCK_BUFFER": 1, "MAX_BLOCK_COST_CLVM": 11000000000})
-    nodes, wallets = await async_gen.__anext__()
+    nodes, wallets, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     server_1 = full_node_1.full_node.server
     wallet_a = bt.get_pool_wallet_tool()
     wallet_receiver = WalletTool(full_node_1.full_node.constants)
-    yield full_node_1, server_1, wallet_a, wallet_receiver
+    yield full_node_1, server_1, wallet_a, wallet_receiver, bt
 
     async for _ in async_gen:
         yield _
 
 
 @pytest_asyncio.fixture(scope="function")
-async def wallet_node_starting_height(self_hostname):
-    async for _ in setup_node_and_wallet(test_constants, self_hostname, starting_height=100):
-        yield _
-
-
-@pytest_asyncio.fixture(scope="function")
-async def wallet_nodes_vanillanet(bt, db_version):
-    async_gen = setup_simulators_and_wallets(2, 1, {"NETWORK_TYPE": 0}, db_version=db_version)
-    nodes, wallets = await async_gen.__anext__()
+async def wallet_nodes_vanillanet(db_version):
+    async_gen = setup_simulators_and_wallets(2, 1, {}, db_version=db_version)
+    nodes, wallets, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     full_node_2 = nodes[1]
     server_1 = full_node_1.full_node.server
     server_2 = full_node_2.full_node.server
     wallet_a = bt.get_pool_wallet_tool()
     wallet_receiver = WalletTool(full_node_1.full_node.constants)
-    yield full_node_1, full_node_2, server_1, server_2, wallet_a, wallet_receiver
+    yield full_node_1, full_node_2, server_1, server_2, wallet_a, wallet_receiver, bt
 
     async for _ in async_gen:
         yield _
@@ -397,9 +408,9 @@ async def wallet_and_node():
 
 
 @pytest_asyncio.fixture(scope="function")
-async def one_node_one_block(bt, wallet_a):
+async def one_node_one_block(wallet_a):
     async_gen = setup_simulators_and_wallets(1, 0, {})
-    nodes, _ = await async_gen.__anext__()
+    nodes, _, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     server_1 = full_node_1.full_node.server
 
@@ -419,16 +430,16 @@ async def one_node_one_block(bt, wallet_a):
 
     await time_out_assert(60, node_height_at_least, True, full_node_1, blocks[-1].height)
 
-    yield full_node_1, server_1
+    yield full_node_1, server_1, bt
 
     async for _ in async_gen:
         yield _
 
 
 @pytest_asyncio.fixture(scope="function")
-async def two_nodes_one_block(bt, wallet_a):
+async def two_nodes_one_block(wallet_a):
     async_gen = setup_simulators_and_wallets(2, 0, {})
-    nodes, _ = await async_gen.__anext__()
+    nodes, _, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     full_node_2 = nodes[1]
     server_1 = full_node_1.full_node.server
@@ -450,7 +461,7 @@ async def two_nodes_one_block(bt, wallet_a):
 
     await time_out_assert(60, node_height_at_least, True, full_node_1, blocks[-1].height)
 
-    yield full_node_1, full_node_2, server_1, server_2
+    yield full_node_1, full_node_2, server_1, server_2, bt
 
     async for _ in async_gen:
         yield _
@@ -458,19 +469,31 @@ async def two_nodes_one_block(bt, wallet_a):
 
 @pytest_asyncio.fixture(scope="function")
 async def farmer_one_harvester(tmp_path: Path, bt: BlockTools) -> AsyncIterator[Tuple[List[Service], Service]]:
-    async for _ in setup_farmer_multi_harvester(bt, 1, tmp_path, test_constants):
+    async for _ in setup_farmer_multi_harvester(bt, 1, tmp_path, test_constants, start_services=True):
         yield _
 
 
 @pytest_asyncio.fixture(scope="function")
-async def farmer_two_harvester(tmp_path: Path, bt: BlockTools) -> AsyncIterator[Tuple[List[Service], Service]]:
-    async for _ in setup_farmer_multi_harvester(bt, 2, tmp_path, test_constants):
+async def farmer_one_harvester_not_started(
+    tmp_path: Path, bt: BlockTools
+) -> AsyncIterator[Tuple[List[Service], Service]]:
+    async for _ in setup_farmer_multi_harvester(bt, 1, tmp_path, test_constants, start_services=False):
         yield _
 
 
 @pytest_asyncio.fixture(scope="function")
-async def farmer_three_harvester(tmp_path: Path, bt: BlockTools) -> AsyncIterator[Tuple[List[Service], Service]]:
-    async for _ in setup_farmer_multi_harvester(bt, 3, tmp_path, test_constants):
+async def farmer_two_harvester_not_started(
+    tmp_path: Path, bt: BlockTools
+) -> AsyncIterator[Tuple[List[Service], Service]]:
+    async for _ in setup_farmer_multi_harvester(bt, 2, tmp_path, test_constants, start_services=False):
+        yield _
+
+
+@pytest_asyncio.fixture(scope="function")
+async def farmer_three_harvester_not_started(
+    tmp_path: Path, bt: BlockTools
+) -> AsyncIterator[Tuple[List[Service], Service]]:
+    async for _ in setup_farmer_multi_harvester(bt, 3, tmp_path, test_constants, start_services=False):
         yield _
 
 
@@ -488,7 +511,7 @@ async def daemon_simulation(bt, get_b_tools, get_b_tools_1):
         connect_to_daemon=True,
         db_version=1,
     ):
-        yield _
+        yield _, get_b_tools, get_b_tools_1
 
 
 @pytest_asyncio.fixture(scope="function")
@@ -529,7 +552,7 @@ async def wallets_prefarm(two_wallet_nodes, self_hostname, trusted):
     """
     farm_blocks = 10
     buffer = 4
-    full_nodes, wallets = two_wallet_nodes
+    full_nodes, wallets, _ = two_wallet_nodes
     full_node_api = full_nodes[0]
     full_node_server = full_node_api.server
     wallet_node_0, wallet_server_0 = wallets[0]
@@ -559,8 +582,8 @@ async def wallets_prefarm(two_wallet_nodes, self_hostname, trusted):
     for i in range(0, buffer):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32(token_bytes(nbytes=32))))
 
-    await time_out_assert(10, wallet_is_synced, True, wallet_node_0, full_node_api)
-    await time_out_assert(10, wallet_is_synced, True, wallet_node_1, full_node_api)
+    await time_out_assert(30, wallet_is_synced, True, wallet_node_0, full_node_api)
+    await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_api)
 
     return wallet_node_0, wallet_node_1, full_node_api
 
@@ -583,3 +606,33 @@ async def setup_sim():
     sim_client = SimClient(sim)
     await sim.farm_block()
     return sim, sim_client
+
+
+@pytest.fixture(scope="function")
+def tmp_chinilla_root(tmp_path):
+    """
+    Create a temp directory and populate it with an empty chinilla_root directory.
+    """
+    path: Path = tmp_path / "chinilla_root"
+    path.mkdir(parents=True, exist_ok=True)
+    return path
+
+
+@pytest.fixture(scope="function")
+def root_path_populated_with_config(tmp_chinilla_root) -> Path:
+    """
+    Create a temp chinilla_root directory and populate it with a default config.yaml.
+    Returns the chinilla_root path.
+    """
+    root_path: Path = tmp_chinilla_root
+    create_default_chinilla_config(root_path)
+    return root_path
+
+
+@pytest.fixture(scope="function")
+def config_with_address_prefix(root_path_populated_with_config: Path, prefix: str) -> Dict[str, Any]:
+    updated_config: Dict[str, Any] = {}
+    with lock_and_load_config(root_path_populated_with_config, "config.yaml") as config:
+        if prefix is not None:
+            config["network_overrides"]["config"][config["selected_network"]]["address_prefix"] = prefix
+    return config
