@@ -7,15 +7,15 @@ from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes, serialization
 
-from chinilla.protocols.shared_protocol import protocol_version
+from chinilla.protocols.shared_protocol import capabilities, protocol_version
 from chinilla.server.outbound_message import NodeType
 from chinilla.server.server import ChinillaServer, ssl_context_for_client
 from chinilla.server.ws_connection import WSChinillaConnection
+from chinilla.simulator.time_out_assert import time_out_assert
 from chinilla.ssl.create_ssl import generate_ca_signed_cert
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
 from chinilla.util.ints import uint16
-from tests.time_out_assert import time_out_assert
 
 log = logging.getLogger(__name__)
 
@@ -68,6 +68,7 @@ async def add_dummy_connection(
         peer_id,
         100,
         30,
+        local_capabilities_for_handshake=capabilities,
     )
     await wsc.perform_handshake(server._network_id, protocol_version, dummy_port, NodeType.FULL_NODE)
     return incoming_queue, peer_id
