@@ -2,6 +2,7 @@ import logging
 import pathlib
 
 import pytest
+from blspy import G1Element
 from clvm_tools import binutils
 
 from chinilla.consensus.condition_costs import ConditionCost
@@ -47,7 +48,7 @@ def large_block_generator(size):
 
 class TestCostCalculation:
     @pytest.mark.asyncio
-    async def test_basics(self, softfork_height, bt):
+    async def test_basics(self, bt):
         wallet_tool = bt.get_pool_wallet_tool()
         ph = wallet_tool.get_new_puzzlehash()
         num_blocks = 3
@@ -73,7 +74,6 @@ class TestCostCalculation:
             test_constants.MAX_BLOCK_COST_CLVM,
             cost_per_byte=test_constants.COST_PER_BYTE,
             mempool_mode=False,
-            height=softfork_height,
         )
 
         assert npc_result.error is None
@@ -106,7 +106,7 @@ class TestCostCalculation:
         )
 
     @pytest.mark.asyncio
-    async def test_mempool_mode(self, softfork_height, bt):
+    async def test_mempool_mode(self, bt):
         wallet_tool = bt.get_pool_wallet_tool()
         ph = wallet_tool.get_new_puzzlehash()
 
@@ -131,7 +131,7 @@ class TestCostCalculation:
         pk = bytes.fromhex(
             "88bc9360319e7c54ab42e19e974288a2d7a817976f7633f4b43f36ce72074e59c4ab8ddac362202f3e366f0aebbb6280"
         )
-        puzzle = p2_delegated_puzzle_or_hidden_puzzle.puzzle_for_pk(pk)
+        puzzle = p2_delegated_puzzle_or_hidden_puzzle.puzzle_for_pk(G1Element.from_bytes(pk))
         disassembly = binutils.disassemble(puzzle)
         program = SerializedProgram.from_bytes(
             binutils.assemble(
@@ -145,7 +145,6 @@ class TestCostCalculation:
             test_constants.MAX_BLOCK_COST_CLVM,
             cost_per_byte=test_constants.COST_PER_BYTE,
             mempool_mode=True,
-            height=softfork_height,
         )
         assert npc_result.error is not None
         npc_result = get_name_puzzle_conditions(
@@ -153,7 +152,6 @@ class TestCostCalculation:
             test_constants.MAX_BLOCK_COST_CLVM,
             cost_per_byte=test_constants.COST_PER_BYTE,
             mempool_mode=False,
-            height=softfork_height,
         )
         assert npc_result.error is None
 
@@ -164,7 +162,7 @@ class TestCostCalculation:
         assert error is None
 
     @pytest.mark.asyncio
-    async def test_clvm_mempool_mode(self, softfork_height):
+    async def test_clvm_mempool_mode(self):
         block = Program.from_bytes(bytes(SMALL_BLOCK_GENERATOR.program))
         disassembly = binutils.disassemble(block)
         # this is a valid generator program except the first clvm
@@ -185,13 +183,12 @@ class TestCostCalculation:
             test_constants.MAX_BLOCK_COST_CLVM,
             cost_per_byte=test_constants.COST_PER_BYTE,
             mempool_mode=False,
-            height=softfork_height,
         )
         assert npc_result.error is None
 
     @pytest.mark.asyncio
     @pytest.mark.benchmark
-    async def test_tx_generator_speed(self, request, softfork_height):
+    async def test_tx_generator_speed(self, request):
         LARGE_BLOCK_COIN_CONSUMED_COUNT = 687
         generator_bytes = large_block_generator(LARGE_BLOCK_COIN_CONSUMED_COUNT)
         program = SerializedProgram.from_bytes(generator_bytes)
@@ -203,14 +200,13 @@ class TestCostCalculation:
                 test_constants.MAX_BLOCK_COST_CLVM,
                 cost_per_byte=test_constants.COST_PER_BYTE,
                 mempool_mode=False,
-                height=softfork_height,
             )
 
         assert npc_result.error is None
         assert len(npc_result.conds.spends) == LARGE_BLOCK_COIN_CONSUMED_COUNT
 
     @pytest.mark.asyncio
-    async def test_clvm_max_cost(self, softfork_height):
+    async def test_clvm_max_cost(self):
 
         block = Program.from_bytes(bytes(SMALL_BLOCK_GENERATOR.program))
         disassembly = binutils.disassemble(block)
@@ -230,7 +226,6 @@ class TestCostCalculation:
             10000000,
             cost_per_byte=0,
             mempool_mode=False,
-            height=softfork_height,
         )
 
         assert npc_result.error is not None
@@ -238,9 +233,7 @@ class TestCostCalculation:
 
         # raise the max cost to make sure this passes
         # ensure we pass if the program does not exceeds the cost
-        npc_result = get_name_puzzle_conditions(
-            generator, 23000000, cost_per_byte=0, mempool_mode=False, height=softfork_height
-        )
+        npc_result = get_name_puzzle_conditions(generator, 23000000, cost_per_byte=0, mempool_mode=False)
 
         assert npc_result.error is None
         assert npc_result.cost > 10000000
@@ -252,7 +245,9 @@ class TestCostCalculation:
         public_key = bytes.fromhex(
             "af949b78fa6a957602c3593a3d6cb7711e08720415dad83" "1ab18adacaa9b27ec3dda508ee32e24bc811c0abc5781ae21"
         )
-        puzzle_program = SerializedProgram.from_bytes(p2_delegated_puzzle_or_hidden_puzzle.puzzle_for_pk(public_key))
+        puzzle_program = SerializedProgram.from_bytes(
+            p2_delegated_puzzle_or_hidden_puzzle.puzzle_for_pk(G1Element.from_bytes(public_key))
+        )
         conditions = binutils.assemble(
             "((51 0x699eca24f2b6f4b25b16f7a418d0dc4fc5fce3b9145aecdda184158927738e3e 10)"
             " (51 0x847bb2385534070c39a39cc5dfdc7b35e2db472dc0ab10ab4dec157a2178adbf 0x00cbba106df6))"
