@@ -9,27 +9,28 @@ from chinilla.full_node.signage_point import SignagePoint
 from chinilla.protocols import full_node_protocol
 from chinilla.rpc.full_node_rpc_api import FullNodeRpcApi
 from chinilla.rpc.full_node_rpc_client import FullNodeRpcClient
-from chinilla.rpc.rpc_server import NodeType, start_rpc_server
+from chinilla.rpc.rpc_server import start_rpc_server
+from chinilla.server.outbound_message import NodeType
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
 from chinilla.types.full_block import FullBlock
 from chinilla.types.spend_bundle import SpendBundle
 from chinilla.types.unfinished_block import UnfinishedBlock
 from chinilla.util.hash import std_hash
 from chinilla.util.ints import uint8, uint16
-from tests.block_tools import get_signage_point
+from chinilla.simulator.block_tools import get_signage_point
 from tests.blockchain.blockchain_test_utils import _validate_and_add_block
 from tests.connection_utils import connect_and_get_peer
 from tests.setup_nodes import test_constants
-from tests.time_out_assert import time_out_assert
+from chinilla.simulator.time_out_assert import time_out_assert
 from tests.util.rpc import validate_get_routes
-from tests.wallet_tools import WalletTool
+from chinilla.simulator.wallet_tools import WalletTool
 
 
 class TestRpc:
     @pytest.mark.asyncio
-    async def test1(self, two_nodes_sim_and_wallets, bt, self_hostname):
+    async def test1(self, two_nodes_sim_and_wallets, self_hostname):
         num_blocks = 5
-        nodes, _ = two_nodes_sim_and_wallets
+        nodes, _, bt = two_nodes_sim_and_wallets
         full_node_api_1, full_node_api_2 = nodes
         server_1 = full_node_api_1.full_node.server
         server_2 = full_node_api_2.full_node.server
@@ -186,6 +187,31 @@ class TestRpc:
             assert len(await client.get_coin_records_by_puzzle_hash(ph, True, 0, blocks[-1].height + 1)) == 2
             assert len(await client.get_coin_records_by_puzzle_hash(ph, True, 0, 1)) == 0
 
+            coin_records = await client.get_coin_records_by_puzzle_hash(ph, False)
+
+            coin_spends = []
+
+            # Spend 3 coins using standard transaction
+            for i in range(3):
+                spend_bundle = wallet.generate_signed_transaction(
+                    coin_records[i].coin.amount, ph_receiver, coin_records[i].coin
+                )
+                await client.push_tx(spend_bundle)
+                coin_spends = coin_spends + spend_bundle.coin_spends
+                await time_out_assert(
+                    5, full_node_api_1.full_node.mempool_manager.get_spendbundle, spend_bundle, spend_bundle.name()
+                )
+
+            await full_node_api_1.farm_new_transaction_block(FarmNewBlockProtocol(ph_2))
+            block: FullBlock = (await full_node_api_1.get_all_full_blocks())[-1]
+
+            assert len(block.transactions_generator_ref_list) > 0  # compression has occurred
+
+            block_spends = await client.get_block_spends(block.header_hash)
+
+            assert len(block_spends) == 3
+            assert block_spends == coin_spends
+
             memo = 32 * b"\f"
 
             for i in range(2):
@@ -244,7 +270,7 @@ class TestRpc:
             blocks: List[FullBlock] = await client.get_blocks(0, 5)
             assert len(blocks) == 5
 
-            await full_node_api_1.reorg_from_index_to_new_index(ReorgProtocol(2, 55, bytes([0x2] * 32)))
+            await full_node_api_1.reorg_from_index_to_new_index(ReorgProtocol(2, 55, bytes([0x2] * 32), None))
             new_blocks_0: List[FullBlock] = await client.get_blocks(0, 5)
             assert len(new_blocks_0) == 7
 
@@ -262,8 +288,8 @@ class TestRpc:
             await rpc_cleanup()
 
     @pytest.mark.asyncio
-    async def test_signage_points(self, two_nodes_sim_and_wallets, empty_blockchain, bt):
-        nodes, _ = two_nodes_sim_and_wallets
+    async def test_signage_points(self, two_nodes_sim_and_wallets, empty_blockchain):
+        nodes, _, bt = two_nodes_sim_and_wallets
         full_node_api_1, full_node_api_2 = nodes
         server_1 = full_node_api_1.full_node.server
         server_2 = full_node_api_2.full_node.server
