@@ -10,7 +10,7 @@ from tests.util.db_connection import DBConnection
 
 
 async def increment_counter(db_wrapper: DBWrapper2) -> None:
-    async with db_wrapper.write_db() as connection:
+    async with db_wrapper.writer_maybe_transaction() as connection:
         async with connection.execute("SELECT value FROM counter") as cursor:
             row = await cursor.fetchone()
 
@@ -24,20 +24,18 @@ async def increment_counter(db_wrapper: DBWrapper2) -> None:
 
 
 async def sum_counter(db_wrapper: DBWrapper2, output: List[int]) -> None:
-    async with db_wrapper.read_db() as connection:
+    async with db_wrapper.reader_no_transaction() as connection:
         async with connection.execute("SELECT value FROM counter") as cursor:
             row = await cursor.fetchone()
 
         assert row is not None
         [value] = row
 
-        for i in range(5):
-            await asyncio.sleep(0)
         output.append(value)
 
 
 async def setup_table(db: DBWrapper2) -> None:
-    async with db.write_db() as conn:
+    async with db.writer_maybe_transaction() as conn:
         await conn.execute("CREATE TABLE counter(value INTEGER NOT NULL)")
         await conn.execute("INSERT INTO counter(value) VALUES(0)")
 
@@ -62,7 +60,7 @@ async def test_concurrent_writers(acquire_outside: bool) -> None:
 
         async with contextlib.AsyncExitStack() as exit_stack:
             if acquire_outside:
-                await exit_stack.enter_async_context(db_wrapper.write_db())
+                await exit_stack.enter_async_context(db_wrapper.writer_maybe_transaction())
 
             tasks = []
             for index in range(concurrent_task_count):
@@ -71,7 +69,7 @@ async def test_concurrent_writers(acquire_outside: bool) -> None:
 
         await asyncio.wait_for(asyncio.gather(*tasks), timeout=None)
 
-        async with db_wrapper.read_db() as connection:
+        async with db_wrapper.reader_no_transaction() as connection:
             async with connection.execute("SELECT value FROM counter") as cursor:
                 row = await cursor.fetchone()
 
@@ -85,14 +83,14 @@ async def test_concurrent_writers(acquire_outside: bool) -> None:
 async def test_writers_nests() -> None:
     async with DBConnection(2) as db_wrapper:
         await setup_table(db_wrapper)
-        async with db_wrapper.write_db() as conn1:
+        async with db_wrapper.writer_maybe_transaction() as conn1:
             async with conn1.execute("SELECT value FROM counter") as cursor:
                 value = await get_value(cursor)
-            async with db_wrapper.write_db() as conn2:
+            async with db_wrapper.writer_maybe_transaction() as conn2:
                 assert conn1 == conn2
                 value += 1
                 await conn2.execute("UPDATE counter SET value = :value", {"value": value})
-                async with db_wrapper.write_db() as conn3:
+                async with db_wrapper.writer_maybe_transaction() as conn3:
                     assert conn1 == conn3
                     async with conn3.execute("SELECT value FROM counter") as cursor:
                         value = await get_value(cursor)
@@ -105,12 +103,12 @@ async def test_partial_failure() -> None:
     values = []
     async with DBConnection(2) as db_wrapper:
         await setup_table(db_wrapper)
-        async with db_wrapper.write_db() as conn1:
+        async with db_wrapper.writer() as conn1:
             await conn1.execute("UPDATE counter SET value = 42")
             async with conn1.execute("SELECT value FROM counter") as cursor:
                 values.append(await get_value(cursor))
             try:
-                async with db_wrapper.write_db() as conn2:
+                async with db_wrapper.writer() as conn2:
                     await conn2.execute("UPDATE counter SET value = 1337")
                     async with conn1.execute("SELECT value FROM counter") as cursor:
                         values.append(await get_value(cursor))
@@ -132,10 +130,10 @@ async def test_readers_nests() -> None:
     async with DBConnection(2) as db_wrapper:
         await setup_table(db_wrapper)
 
-        async with db_wrapper.read_db() as conn1:
-            async with db_wrapper.read_db() as conn2:
+        async with db_wrapper.reader_no_transaction() as conn1:
+            async with db_wrapper.reader_no_transaction() as conn2:
                 assert conn1 == conn2
-                async with db_wrapper.read_db() as conn3:
+                async with db_wrapper.reader_no_transaction() as conn3:
                     assert conn1 == conn3
                     async with conn3.execute("SELECT value FROM counter") as cursor:
                         value = await get_value(cursor)
@@ -148,10 +146,10 @@ async def test_readers_nests_writer() -> None:
     async with DBConnection(2) as db_wrapper:
         await setup_table(db_wrapper)
 
-        async with db_wrapper.write_db() as conn1:
-            async with db_wrapper.read_db() as conn2:
+        async with db_wrapper.writer_maybe_transaction() as conn1:
+            async with db_wrapper.reader_no_transaction() as conn2:
                 assert conn1 == conn2
-                async with db_wrapper.write_db() as conn3:
+                async with db_wrapper.writer_maybe_transaction() as conn3:
                     assert conn1 == conn3
                     async with conn3.execute("SELECT value FROM counter") as cursor:
                         value = await get_value(cursor)
@@ -169,14 +167,14 @@ async def test_concurrent_readers(acquire_outside: bool) -> None:
     async with DBConnection(2) as db_wrapper:
         await setup_table(db_wrapper)
 
-        async with db_wrapper.write_db() as connection:
+        async with db_wrapper.writer_maybe_transaction() as connection:
             await connection.execute("UPDATE counter SET value = 1")
 
         concurrent_task_count = 200
 
         async with contextlib.AsyncExitStack() as exit_stack:
             if acquire_outside:
-                await exit_stack.enter_async_context(db_wrapper.read_db())
+                await exit_stack.enter_async_context(db_wrapper.reader_no_transaction())
 
             tasks = []
             values: List[int] = []
@@ -199,14 +197,14 @@ async def test_mixed_readers_writers(acquire_outside: bool) -> None:
     async with DBConnection(2) as db_wrapper:
         await setup_table(db_wrapper)
 
-        async with db_wrapper.write_db() as connection:
+        async with db_wrapper.writer_maybe_transaction() as connection:
             await connection.execute("UPDATE counter SET value = 1")
 
         concurrent_task_count = 200
 
         async with contextlib.AsyncExitStack() as exit_stack:
             if acquire_outside:
-                await exit_stack.enter_async_context(db_wrapper.read_db())
+                await exit_stack.enter_async_context(db_wrapper.reader_no_transaction())
 
             tasks = []
             values: List[int] = []
@@ -216,6 +214,7 @@ async def test_mixed_readers_writers(acquire_outside: bool) -> None:
                     tasks.append(task)
                 task = asyncio.create_task(sum_counter(db_wrapper, values))
                 tasks.append(task)
+                await asyncio.sleep(0.001)
 
         await asyncio.wait_for(asyncio.gather(*tasks), timeout=None)
 
