@@ -538,10 +538,22 @@ request_block_header = wallet_protocol.RequestBlockHeader(
     uint32(3562957314),
 )
 
+request_block_headers = wallet_protocol.RequestBlockHeaders(
+    uint32(1234970524),
+    uint32(234653234),
+    False,
+)
+
 respond_header_block = wallet_protocol.RespondBlockHeader(
     header_block,
 )
 
+respond_block_headers = wallet_protocol.RespondBlockHeaders(
+    uint32(923662371),
+    uint32(992357623),
+    [header_block],
+)
+
 reject_header_request = wallet_protocol.RejectHeaderRequest(
     uint32(17867635),
 )
@@ -598,6 +610,11 @@ reject_header_blocks = wallet_protocol.RejectHeaderBlocks(
     uint32(2908717391),
 )
 
+reject_block_headers = wallet_protocol.RejectBlockHeaders(
+    uint32(543373229),
+    uint32(2347869036),
+)
+
 respond_header_blocks = wallet_protocol.RespondHeaderBlocks(
     uint32(4130100992),
     uint32(17664086),
