@@ -5,7 +5,7 @@ import pytest
 
 from chinilla.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
 from chinilla.full_node.mempool_manager import MempoolManager
-from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol
+from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
 from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
@@ -17,8 +17,8 @@ from chinilla.wallet.cat_wallet.cat_wallet import CATWallet
 from chinilla.wallet.puzzles.cat_loader import CAT_MOD
 from chinilla.wallet.transaction_record import TransactionRecord
 from chinilla.wallet.wallet_info import WalletInfo
-from tests.pools.test_pool_rpc import wallet_is_synced
-from tests.time_out_assert import time_out_assert
+from chinilla.simulator.time_out_assert import time_out_assert
+from tests.util.wallet_is_synced import wallet_is_synced
 
 
 async def tx_in_pool(mempool: MempoolManager, tx_id: bytes32):
@@ -36,7 +36,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_cat_creation(self, self_hostname, two_wallet_nodes, trusted):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -60,8 +60,8 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -80,9 +80,9 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 100)
-        await time_out_assert(15, cat_wallet.get_spendable_balance, 100)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_spendable_balance, 100)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 100)
 
         # Test migration
         all_lineage = await cat_wallet.lineage_store.get_all_lineage_proofs()
@@ -98,10 +98,16 @@ class TestCATWallet:
         assert new_cat_wallet.cat_info.my_tail == cat_wallet.cat_info.my_tail
         assert await cat_wallet.lineage_store.get_all_lineage_proofs() == all_lineage
 
+        height = full_node_api.full_node.blockchain.get_peak_height()
+        await full_node_api.reorg_from_index_to_new_index(
+            ReorgProtocol(height - num_blocks - 1, height + 1, 32 * b"1", None)
+        )
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 0)
+
     @pytest.mark.asyncio
     async def test_cat_creation_unique_lineage_store(self, self_hostname, two_wallet_nodes):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, wallet_server = wallets[0]
@@ -121,8 +127,8 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet_1: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -145,7 +151,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_cat_spend(self, self_hostname, two_wallet_nodes, trusted):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -174,7 +180,7 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -189,8 +195,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 100)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 100)
 
         assert cat_wallet.cat_info.limitations_program_hash is not None
         asset_id = cat_wallet.get_asset_id()
@@ -212,15 +218,15 @@ class TestCATWallet:
             if tx_record.wallet_id is cat_wallet.id():
                 assert tx_record.to_puzzle_hash == cat_2_hash
 
-        await time_out_assert(15, cat_wallet.get_pending_change_balance, 40)
+        await time_out_assert(20, cat_wallet.get_pending_change_balance, 40)
 
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"\0"))
 
         await time_out_assert(30, wallet.get_confirmed_balance, funds - 101)
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 40)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 40)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 40)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 40)
 
         await time_out_assert(30, cat_wallet_2.get_confirmed_balance, 60)
         await time_out_assert(30, cat_wallet_2.get_unconfirmed_balance, 60)
@@ -233,11 +239,14 @@ class TestCATWallet:
                 15, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx_record.spend_bundle.name()
             )
 
-        for i in range(1, num_blocks):
-            await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 55)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 55)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 55)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 55)
+
+        height = full_node_api.full_node.blockchain.get_peak_height()
+        await full_node_api.reorg_from_index_to_new_index(ReorgProtocol(height - 1, height + 1, 32 * b"1", None))
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 40)
 
     @pytest.mark.parametrize(
         "trusted",
@@ -246,7 +255,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_get_wallet_for_asset_id(self, self_hostname, two_wallet_nodes, trusted):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -270,7 +279,7 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -299,7 +308,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_cat_doesnt_see_eve(self, self_hostname, two_wallet_nodes, trusted):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -328,7 +337,7 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -342,8 +351,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 100)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 100)
 
         assert cat_wallet.cat_info.limitations_program_hash is not None
         asset_id = cat_wallet.get_asset_id()
@@ -368,11 +377,11 @@ class TestCATWallet:
         await time_out_assert(30, wallet.get_confirmed_balance, funds - 101)
         await time_out_assert(30, wallet.get_unconfirmed_balance, funds - 101)
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 40)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 40)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 40)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 40)
 
-        await time_out_assert(15, cat_wallet_2.get_confirmed_balance, 60)
-        await time_out_assert(15, cat_wallet_2.get_unconfirmed_balance, 60)
+        await time_out_assert(20, cat_wallet_2.get_confirmed_balance, 60)
+        await time_out_assert(20, cat_wallet_2.get_unconfirmed_balance, 60)
 
         cc2_ph = await cat_wallet_2.get_new_cat_puzzle_hash()
         tx_record = await wallet.wallet_state_manager.main_wallet.generate_signed_transaction(10, cc2_ph, 0)
@@ -390,10 +399,10 @@ class TestCATWallet:
             all_txs = await wsm.tx_store.get_all_transactions_for_wallet(id)
             return len(list(filter(lambda tx: tx.amount == 10, all_txs)))
 
-        await time_out_assert(15, query_and_assert_transactions, 0, wsm, id)
-        await time_out_assert(15, wsm.get_confirmed_balance_for_wallet, 60, id)
-        await time_out_assert(15, cat_wallet_2.get_confirmed_balance, 60)
-        await time_out_assert(15, cat_wallet_2.get_unconfirmed_balance, 60)
+        await time_out_assert(20, query_and_assert_transactions, 0, wsm, id)
+        await time_out_assert(20, wsm.get_confirmed_balance_for_wallet, 60, id)
+        await time_out_assert(20, cat_wallet_2.get_confirmed_balance, 60)
+        await time_out_assert(20, cat_wallet_2.get_unconfirmed_balance, 60)
 
     @pytest.mark.parametrize(
         "trusted",
@@ -402,7 +411,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_cat_spend_multiple(self, self_hostname, three_wallet_nodes, trusted):
         num_blocks = 3
-        full_nodes, wallets = three_wallet_nodes
+        full_nodes, wallets, _ = three_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node_0, wallet_server_0 = wallets[0]
@@ -432,7 +441,7 @@ class TestCATWallet:
             [calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, num_blocks)]
         )
 
-        await time_out_assert(15, wallet_0.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet_0.get_confirmed_balance, funds)
 
         async with wallet_node_0.wallet_state_manager.lock:
             cat_wallet_0: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -446,8 +455,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet_0.get_confirmed_balance, 100)
-        await time_out_assert(15, cat_wallet_0.get_unconfirmed_balance, 100)
+        await time_out_assert(20, cat_wallet_0.get_confirmed_balance, 100)
+        await time_out_assert(20, cat_wallet_0.get_unconfirmed_balance, 100)
 
         assert cat_wallet_0.cat_info.limitations_program_hash is not None
         asset_id = cat_wallet_0.get_asset_id()
@@ -475,8 +484,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet_0.get_confirmed_balance, 20)
-        await time_out_assert(15, cat_wallet_0.get_unconfirmed_balance, 20)
+        await time_out_assert(20, cat_wallet_0.get_confirmed_balance, 20)
+        await time_out_assert(20, cat_wallet_0.get_unconfirmed_balance, 20)
 
         await time_out_assert(30, cat_wallet_1.get_confirmed_balance, 60)
         await time_out_assert(30, cat_wallet_1.get_unconfirmed_balance, 60)
@@ -503,8 +512,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet_0.get_confirmed_balance, 55)
-        await time_out_assert(15, cat_wallet_0.get_unconfirmed_balance, 55)
+        await time_out_assert(20, cat_wallet_0.get_confirmed_balance, 55)
+        await time_out_assert(20, cat_wallet_0.get_unconfirmed_balance, 55)
 
         await time_out_assert(30, cat_wallet_1.get_confirmed_balance, 45)
         await time_out_assert(30, cat_wallet_1.get_unconfirmed_balance, 45)
@@ -543,7 +552,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_cat_max_amount_send(self, self_hostname, two_wallet_nodes, trusted):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -571,7 +580,7 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -585,8 +594,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 100000)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 100000)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 100000)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 100000)
 
         assert cat_wallet.cat_info.limitations_program_hash is not None
 
@@ -624,7 +633,7 @@ class TestCATWallet:
                     return False
             return True
 
-        await time_out_assert(15, check_all_there, True)
+        await time_out_assert(20, check_all_there, True)
         await asyncio.sleep(5)
         max_sent_amount = await cat_wallet.get_max_send_amount()
 
@@ -675,7 +684,7 @@ class TestCATWallet:
     @pytest.mark.asyncio
     async def test_cat_hint(self, self_hostname, two_wallet_nodes, trusted, autodiscovery):
         num_blocks = 3
-        full_nodes, wallets = two_wallet_nodes
+        full_nodes, wallets, _ = two_wallet_nodes
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -706,7 +715,7 @@ class TestCATWallet:
             ]
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
 
         async with wallet_node.wallet_state_manager.lock:
             cat_wallet: CATWallet = await CATWallet.create_new_cat_wallet(
@@ -720,8 +729,8 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(32 * b"0"))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 100)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 100)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 100)
         assert cat_wallet.cat_info.limitations_program_hash is not None
 
         cat_2_hash = await wallet2.get_new_puzzlehash()
@@ -737,18 +746,18 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 40)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 40)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 40)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 40)
 
         async def check_wallets(node):
             return len(node.wallet_state_manager.wallets.keys())
 
         if autodiscovery:
             # Autodiscovery enabled: test that wallet was created at this point
-            await time_out_assert(10, check_wallets, 2, wallet_node_2)
+            await time_out_assert(20, check_wallets, 2, wallet_node_2)
         else:
             # Autodiscovery disabled: test that no wallet was created
-            await time_out_assert(10, check_wallets, 1, wallet_node_2)
+            await time_out_assert(20, check_wallets, 1, wallet_node_2)
 
         # Then we update the wallet's default CATs
         wallet_node_2.wallet_state_manager.default_cats = {
@@ -772,11 +781,11 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 30)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 30)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 30)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 30)
 
         # Now we check that another wallet WAS created, even if autodiscovery was disabled
-        await time_out_assert(10, check_wallets, 2, wallet_node_2)
+        await time_out_assert(20, check_wallets, 2, wallet_node_2)
         cat_wallet_2 = wallet_node_2.wallet_state_manager.wallets[2]
 
         # Previous balance + balance that triggered creation in case of disabled autodiscovery
@@ -795,5 +804,5 @@ class TestCATWallet:
         for i in range(1, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
 
-        await time_out_assert(15, cat_wallet.get_confirmed_balance, 35)
-        await time_out_assert(15, cat_wallet.get_unconfirmed_balance, 35)
+        await time_out_assert(20, cat_wallet.get_confirmed_balance, 35)
+        await time_out_assert(20, cat_wallet.get_unconfirmed_balance, 35)
